@@ -1,6 +1,6 @@
 /* db.c: an external database to avoid filesystem lookups.
 
-   Copyright 1994, 1995, 1996, 1997, 2008 Karl Berry.
+   Copyright 1994, 1995, 1996, 1997, 2008, 2009, 2011, 2012 Karl Berry.
    Copyright 1997-2005 Olaf Weber.
 
    This library is free software; you can redistribute it and/or
@@ -43,9 +43,12 @@
 #define DB_NAME_LC "ls-r"
 #endif
 
+/* read ls-R only on WIN32 */
 static const_string db_names[] = {
     DB_NAME,
+#ifndef WIN32
     DB_NAME_LC,
+#endif
     NULL
 };
 
@@ -65,13 +68,13 @@ static boolean
 ignore_dir_p (const_string dirname)
 {
   const_string dot_pos = dirname;
-  
+
   while ((dot_pos = strchr (dot_pos + 1, '.'))) {
     /* If / before and no / after, skip it. */
-    if (IS_DIR_SEP (dot_pos[-1]) && dot_pos[1] && !IS_DIR_SEP (dot_pos[1]))
+    if (IS_DIR_SEP_CH (dot_pos[-1]) && dot_pos[1] && !IS_DIR_SEP_CH (dot_pos[1]))
       return true;
   }
-  
+
   return false;
 }
 
@@ -87,19 +90,32 @@ db_build (kpathsea kpse, hash_table_type *table,  const_string db_filename)
   string top_dir = (string)xmalloc (len + 1);
   string cur_dir = NULL; /* First thing in ls-R might be a filename.  */
   FILE *db_file = fopen (db_filename, FOPEN_R_MODE);
-  
+#if defined(WIN32)
+  string pp;
+#endif
+
   strncpy (top_dir, db_filename, len);
   top_dir[len] = 0;
-  
+
   if (db_file) {
     while ((line = read_line (db_file)) != NULL) {
       len = strlen (line);
 
+#if defined(WIN32)
+      for (pp = line; *pp; pp++) {
+        if (IS_KANJI(pp))
+          pp++;
+        else
+          *pp = TRANSFORM(*pp);
+      }
+#endif
+
       /* A line like `/foo:' = new dir foo.  Allow both absolute (/...)
          and explicitly relative (./...) names here.  It's a kludge to
          pass in the directory name with the trailing : still attached,
          but it doesn't actually hurt.  */
-      if (len > 0 && line[len - 1] == ':' && kpathsea_absolute_p (kpse, line, true)) {
+      if (len > 0 && line[len - 1] == ':'
+          && kpathsea_absolute_p (kpse, line, true)) {
         /* New directory line.  */
         if (!ignore_dir_p (line)) {
           /* If they gave a relative name, prepend full directory name now.  */
@@ -139,7 +155,7 @@ db_build (kpathsea kpse, hash_table_type *table,  const_string db_filename)
     xfclose (db_file, db_filename);
 
     if (file_count == 0) {
-      WARNING1 ("kpathsea: No usable entries in %s", db_filename);
+      WARNING1 ("kpathsea: %s: No usable entries in ls-R", db_filename);
       WARNING ("kpathsea: See the manual for how to generate ls-R");
       db_file = NULL;
     } else {
@@ -181,7 +197,7 @@ kpathsea_db_insert (kpathsea kpse, const_string passed_fname)
   if (kpse->db.buckets) {
     const_string dir_part;
     string fname = xstrdup (passed_fname);
-    string baseptr = (string)xbasename (fname);
+    string baseptr = fname + (xbasename (fname) - fname);
     const_string file_part = xstrdup (baseptr);
 
     *baseptr = '\0';  /* Chop off the filename.  */
@@ -191,32 +207,24 @@ kpathsea_db_insert (kpathsea kpse, const_string passed_fname)
     hash_insert (&(kpse->db), file_part, dir_part);
   }
 }
-
-#if defined(KPSE_COMPAT_API)
-void
-kpse_db_insert (const_string passed_fname)
-{
-    kpathsea_db_insert(kpse_def, passed_fname);
-}
-#endif
 
 /* Return true if FILENAME could be in PATH_ELT, i.e., if the directory
    part of FILENAME matches PATH_ELT.  Have to consider // wildcards, but
    $ and ~ expansion have already been done.  */
-     
+
 static boolean
 match (const_string filename,  const_string path_elt)
 {
   const_string original_filename = filename;
   boolean matched = false;
-  
+
   for (; *filename && *path_elt; filename++, path_elt++) {
     if (FILECHARCASEEQ (*filename, *path_elt)) /* normal character match */
       ;
 
-    else if (IS_DIR_SEP (*path_elt)  /* at // */
-             && original_filename < filename && IS_DIR_SEP (path_elt[-1])) {
-      while (IS_DIR_SEP (*path_elt))
+    else if (IS_DIR_SEP_CH (*path_elt)  /* at // */
+             && original_filename < filename && IS_DIR_SEP_CH (path_elt[-1])) {
+      while (IS_DIR_SEP_CH (*path_elt))
         path_elt++; /* get past second and any subsequent /'s */
       if (*path_elt == 0) {
         /* Trailing //, matches anything. We could make this part of the
@@ -227,7 +235,7 @@ match (const_string filename,  const_string path_elt)
         /* Intermediate //, have to match rest of PATH_ELT.  */
         for (; !matched && *filename; filename++) {
           /* Try matching at each possible character.  */
-          if (IS_DIR_SEP (filename[-1])
+          if (IS_DIR_SEP_CH (filename[-1])
               && FILECHARCASEEQ (*filename, *path_elt))
             matched = match (filename, path_elt);
         }
@@ -239,24 +247,41 @@ match (const_string filename,  const_string path_elt)
     else /* normal character nonmatch, quit */
       break;
   }
-  
+
   /* If we've reached the end of PATH_ELT, check that we're at the last
-     component of FILENAME, we've matched.  */
+     component of FILENAME (that is, no directory separators remaining);
+     only then have we matched.  */
   if (!matched && *path_elt == 0) {
-    /* Probably PATH_ELT ended with `vf' or some such, and FILENAME ends
-       with `vf/ptmr.vf'.  In that case, we'll be at a directory
-       separator.  On the other hand, if PATH_ELT ended with a / (as in
-       `vf/'), FILENAME being the same `vf/ptmr.vf', we'll be at the
-       `p'.  Upshot: if we're at a dir separator in FILENAME, skip it.
-       But if not, that's ok, as long as there are no more dir separators.  */
-    if (IS_DIR_SEP (*filename))
+    /* Typically PATH_ELT ends with, say, `vf', and FILENAME ends with
+       `vf/ptmr.vf'.  In that case, we'll be at the /.  On the other
+       hand, if PATH_ELT ended with a / (as in `vf/'), FILENAME being
+       the same `vf/ptmr.vf', we'll be at the `p'.
+       Upshot: if we're at a dir sep in FILENAME, skip it.  */
+    if (IS_DIR_SEP_CH (*filename))
       filename++;
-      
-    while (*filename && !IS_DIR_SEP (*filename))
-      filename++;
-    matched = *filename == 0;
+
+    /* Here are the basic possibilities for the check on being at the
+       last component:
+       1) PATH_ELT is empty and FILENAME is `ptmr.vf'     => match.
+          (we now have original_filename == filename)
+       2) PATH_ELT is empty and FILENAME is `foo/ptmr.vf' => no match.
+          (we now have original_filename == filename)
+       3) PATH_ELT is `vf/' and FILENAME is `vf/ptmr.vf'
+          (we are now after the / in each)                 => match.
+       4) PATH_ELT is `vf' and FILENAME is `vfoo.ext'
+          (we are now after the f in each)                 => no match.
+       
+       When (the original) PATH_ELT was the empty string, we want to match
+       a FILENAME without dir seps.  (This could be argued, and may never
+       happen in practice, but is the historical behavior.)  */
+    /* if original_filename != filename then original_filename < filename */
+    if (original_filename == filename || IS_DIR_SEP_CH (filename[-1])) {
+      while (*filename && !IS_DIR_SEP_CH (*filename))
+        filename++;
+      matched = *filename == 0;
+    }
   }
-  
+
   return matched;
 }
 
@@ -265,12 +290,12 @@ match (const_string filename,  const_string path_elt)
    That is, the question is whether to try the db for a file looked up
    in PATH_ELT.  If PATH_ELT == ".", for example, the answer is no. If
    PATH_ELT == "/usr/local/lib/texmf/fonts//tfm", the answer is yes.
-   
+
    In practice, ls-R is only needed for lengthy subdirectory
    comparisons, but there's no gain to checking PATH_ELT to see if it is
    a subdir match, since the only way to do that is to do a string
    search in it, which is all we do anyway.  */
-   
+
 static boolean
 elt_in_db (const_string db_dir,  const_string path_elt)
 {
@@ -280,7 +305,7 @@ elt_in_db (const_string db_dir,  const_string path_elt)
     /* If we've matched the entire db directory, it's good.  */
     if (*db_dir == 0)
       found = true;
- 
+
     /* If we've reached the end of PATH_ELT, but not the end of the db
        directory, it's no good.  */
     else if (*path_elt == 0)
@@ -293,7 +318,8 @@ elt_in_db (const_string db_dir,  const_string path_elt)
 /* If ALIAS_FILENAME exists, read it into TABLE.  */
 
 static boolean
-alias_build (kpathsea kpse, hash_table_type *table,  const_string alias_filename)
+alias_build (kpathsea kpse, hash_table_type *table,
+             const_string alias_filename)
 {
   string line, real, alias;
   unsigned count = 0;
@@ -313,7 +339,7 @@ alias_build (kpathsea kpse, hash_table_type *table,  const_string alias_filename
         while (*alias && !ISSPACE (*alias))
           alias++;
         *alias++ = 0;
-        while (*alias && ISSPACE (*alias)) 
+        while (*alias && ISSPACE (*alias))
           alias++;
         /* Is the check for errors strong enough?  Should we warn the user
            for potential errors?  */
@@ -344,7 +370,7 @@ alias_build (kpathsea kpse, hash_table_type *table,  const_string alias_filename
 }
 
 /* Initialize the path for ls-R files, and read them all into the hash
-   table `db'.  If no usable ls-R's are found, set kpse->db.buckets to NULL.  */
+   table `db'.  If no usable ls-R's found, set kpse->db.buckets to NULL.  */
 
 void
 kpathsea_init_db (kpathsea kpse)
@@ -359,7 +385,7 @@ kpathsea_init_db (kpathsea kpse)
   db_path = kpathsea_init_format (kpse, kpse_db_format);
   db_files = kpathsea_all_path_search_list (kpse, db_path, db_names);
   orig_db_files = db_files;
-  
+
   /* Must do this after the path searching (which ends up calling
     kpse_db_search recursively), so kpse->db.buckets stays NULL.  */
   kpse->db = hash_create (DB_HASH_SIZE);
@@ -370,7 +396,7 @@ kpathsea_init_db (kpathsea kpse)
     free (*db_files);
     db_files++;
   }
-  
+
   if (!ok) {
     /* If db can't be built, leave `size' nonzero (so we don't
        rebuild it), but clear `buckets' (so we don't look in it).  */
@@ -403,35 +429,26 @@ kpathsea_init_db (kpathsea kpse)
 
   free (orig_db_files);
 }
-
-#if defined(KPSE_COMPAT_API)
-void
-kpse_init_db (void)
-{
-    kpathsea_init_db(kpse_def);
-}
-#endif
-
 
 /* Avoid doing anything if this PATH_ELT is irrelevant to the databases. */
 str_list_type *
-kpathsea_db_search (kpathsea kpse, const_string name,  
+kpathsea_db_search (kpathsea kpse, const_string name,
                     const_string orig_path_elt, boolean all)
 {
-  string *db_dirs, *orig_dirs, *r;
-  const_string last_slash;
-  string path_elt;
+  string *db_dirs, *orig_dirs;
+  const_string last_slash, path_elt;
+  string temp_str = NULL;
   boolean done;
   unsigned e;
   str_list_type *ret = NULL;
-  string *aliases = NULL;
+  const_string *aliases, *r;
   boolean relevant = false;
-  
+
   /* If we failed to build the database (or if this is the recursive
      call to build the db path), quit.  */
   if (kpse->db.buckets == NULL)
     return NULL;
-  
+
   /* When tex-glyph.c calls us looking for, e.g., dpi600/cmr10.pk, we
      won't find it unless we change NAME to just `cmr10.pk' and append
      `/dpi600' to PATH_ELT.  We are justified in using a literal `/'
@@ -444,10 +461,11 @@ kpathsea_db_search (kpathsea kpse, const_string name,
     string dir_part = (string)xmalloc (len);
     strncpy (dir_part, name, len - 1);
     dir_part[len - 1] = 0;
-    path_elt = concat3 (orig_path_elt, "/", dir_part);
+    path_elt = temp_str = concat3 (orig_path_elt, "/", dir_part);
     name = last_slash + 1;
+    free (dir_part);
   } else
-    path_elt = (string) orig_path_elt;
+    path_elt = orig_path_elt;
 
   /* Don't bother doing any lookups if this `path_elt' isn't covered by
      any of database directories.  We do this not so much because the
@@ -462,10 +480,12 @@ kpathsea_db_search (kpathsea kpse, const_string name,
 
   /* If we have aliases for this name, use them.  */
   if (kpse->alias_db.buckets)
-    aliases = hash_lookup (kpse->alias_db, name);
+    aliases = (const_string *) hash_lookup (kpse->alias_db, name);
+  else
+    aliases = NULL;
 
   if (!aliases) {
-    aliases = XTALLOC1 (string);
+    aliases = XTALLOC1 (const_string);
     aliases[0] = NULL;
   }
   {  /* Push aliases up by one and insert the original name at the front.  */
@@ -473,16 +493,20 @@ kpathsea_db_search (kpathsea kpse, const_string name,
     unsigned len = 1; /* Have NULL element already allocated.  */
     for (r = aliases; *r; r++)
       len++;
-    XRETALLOC (aliases, len + 1, string);
+    /* This is essentially
+    XRETALLOC (aliases, len + 1, const_string);
+       except that MSVC warns without the cast to `void *'.  */
+    aliases = (const_string *) xrealloc ((void *) aliases,
+                                         (len + 1) * sizeof(const_string));
     for (i = len; i > 0; i--) {
       aliases[i] = aliases[i - 1];
     }
-    aliases[0] = (string) name;
+    aliases[0] = name;
   }
 
   done = false;
   for (r = aliases; !done && *r; r++) {
-    string ctry = *r;
+    const_string ctry = *r;
 
     /* We have an ls-R db.  Look up `try'.  */
     orig_dirs = db_dirs = hash_lookup (kpse->db, ctry);
@@ -508,12 +532,12 @@ kpathsea_db_search (kpathsea kpse, const_string name,
         string found = NULL;
         if (kpathsea_readable_file (kpse, db_file)) {
           found = db_file;
-          
+
         } else {
-          string *a;
-          
+          const_string *a;
+
           free (db_file); /* `db_file' wasn't on disk.  */
-          
+
           /* The hit in the DB doesn't exist in disk.  Now try all its
              aliases.  For example, suppose we have a hierarchy on CD,
              thus `mf.bas', but ls-R contains `mf.base'.  Find it anyway.
@@ -528,7 +552,7 @@ kpathsea_db_search (kpathsea kpse, const_string name,
               free (atry);
           }
         }
-          
+
         /* If we have a real file, add it to the list, maybe done.  */
         if (found) {
           str_list_add (ret, found);
@@ -538,7 +562,7 @@ kpathsea_db_search (kpathsea kpse, const_string name,
       } else { /* no match in the db */
         free (db_file);
       }
-      
+
 
       /* On to the next directory, if any.  */
       db_dirs++;
@@ -548,38 +572,30 @@ kpathsea_db_search (kpathsea kpse, const_string name,
     if (orig_dirs && *orig_dirs)
       free (orig_dirs);
   }
-  
-  free (aliases);
-  
-  /* If we had to break up NAME, free the temporary PATH_ELT.  */
-  if (path_elt != orig_path_elt)
-    free (path_elt);
+
+  free ((void *) aliases);
+
+  /* If we had to break up NAME, free the TEMP_STR.  */
+  if (temp_str)
+    free (temp_str);
 
   return ret;
 }
-
-#if defined(KPSE_COMPAT_API)
-str_list_type *
-kpse_db_search (const_string name,  const_string orig_path_elt,
-                boolean all)
-{
-    return kpathsea_db_search (kpse_def, name, orig_path_elt, all);
-}
-#endif
 
 str_list_type *
-kpathsea_db_search_list (kpathsea kpse, const_string* names,  
+kpathsea_db_search_list (kpathsea kpse, const_string* names,
                          const_string path_elt, boolean all)
 {
-  string *db_dirs, *orig_dirs, *r;
+  string *db_dirs, *orig_dirs;
   const_string last_slash, name, path;
+  string temp_str = NULL;
   boolean done;
   unsigned e;
-  string *aliases;
+  const_string *aliases, *r;
   int n;
   str_list_type *ret = NULL;
   boolean relevant = false;
-  
+
   /* If we failed to build the database (or if this is the recursive
      call to build the db path), quit.  */
   if (kpse->db.buckets == NULL)
@@ -597,6 +613,9 @@ kpathsea_db_search_list (kpathsea kpse, const_string* names,
     return NULL;
 
   done = false;
+  ret = XTALLOC1 (str_list_type);
+  *ret = str_list_init ();
+
   /* Handle each name. */
   for (n = 0; !done && names[n]; n++) {
       name = names[n];
@@ -617,115 +636,104 @@ kpathsea_db_search_list (kpathsea kpse, const_string* names,
           string dir_part = (string)xmalloc (len);
           strncpy (dir_part, name, len - 1);
           dir_part[len - 1] = 0;
-          path = concat3 (path_elt, "/", dir_part);
+          path = temp_str = concat3 (path_elt, "/", dir_part);
           name = last_slash + 1;
-          free(dir_part);
+          free (dir_part);
       } else {
           path = path_elt;
       }
-      
+
       /* If we have aliases for this name, use them.  */
       if (kpse->alias_db.buckets)
-          aliases = hash_lookup (kpse->alias_db, name);
+          aliases = (const_string *) hash_lookup (kpse->alias_db, name);
       else
           aliases = NULL;
 
       if (!aliases) {
-          aliases = XTALLOC1 (string);
+          aliases = XTALLOC1 (const_string);
           aliases[0] = NULL;
-      } 
-      {  /* Push aliases up by one and insert the original name at the front.  */
+      }
+      {  /* Push aliases up by one and insert the original name at front.  */
           unsigned i;
           unsigned len = 1; /* Have NULL element already allocated.  */
           for (r = aliases; *r; r++)
               len++;
-          XRETALLOC (aliases, len + 1, string);
+          aliases = (const_string *) xrealloc ((void *) aliases,
+                                               (len + 1) * sizeof(const_string));
           for (i = len; i > 0; i--) {
               aliases[i] = aliases[i - 1];
           }
-          aliases[0] = (string) name;
+          aliases[0] = name;
       }
 
       for (r = aliases; !done && *r; r++) {
-          string ctry = *r;
+          const_string ctry = *r;
 
           /* We have an ls-R db.  Look up `try'.  */
           orig_dirs = db_dirs = hash_lookup (kpse->db, ctry);
-          
-          ret = XTALLOC1 (str_list_type);
-          *ret = str_list_init ();
 
           /* For each filename found, see if it matches the path element.  For
              example, if we have .../cx/cmr10.300pk and .../ricoh/cmr10.300pk,
              and the path looks like .../cx, we don't want the ricoh file.  */
           while (!done && db_dirs && *db_dirs) {
-              string db_file = concat (*db_dirs, ctry);
-              boolean matched = match (db_file, path);
-              
+            string db_file = concat (*db_dirs, ctry);
+            boolean matched = match (db_file, path);
+
 #ifdef KPSE_DEBUG
-              if (KPATHSEA_DEBUG_P (KPSE_DEBUG_SEARCH))
-                DEBUGF3 ("db:match(%s,%s) = %d\n", db_file, path, matched);
+            if (KPATHSEA_DEBUG_P (KPSE_DEBUG_SEARCH))
+              DEBUGF3 ("db:match(%s,%s) = %d\n", db_file, path, matched);
 #endif
 
-              /* We got a hit in the database.  Now see if the file actually
-                 exists, possibly under an alias.  */
-              if (matched) {
-                  string found = NULL;
-                  if (kpathsea_readable_file (kpse, db_file)) {
-                      found = db_file;
-                      
-                  } else {
-                      string *a;
-                      
-                      free (db_file); /* `db_file' wasn't on disk.  */
-                      
-                      /* The hit in the DB doesn't exist in disk.  Now try all its
-                         aliases.  For example, suppose we have a hierarchy on CD,
-                         thus `mf.bas', but ls-R contains `mf.base'.  Find it anyway.
-                         Could probably work around this with aliases, but
-                         this is pretty easy and shouldn't hurt.  The upshot is that
-                         if one of the aliases actually exists, we use that.  */
-                      for (a = aliases + 1; *a && !found; a++) {
-                          string atry = concat (*db_dirs, *a);
-                          if (kpathsea_readable_file (kpse, atry))
-                              found = atry;
-                          else
-                              free (atry);
-                      }
-                  }
-                  
-                  /* If we have a real file, add it to the list, maybe done.  */
-                  if (found) {
-                      str_list_add (ret, found);
-                      if (!all && found)
-                          done = true;
-                  }
-              } else { /* no match in the db */
-                  free (db_file);
+            /* We got a hit in the database.  Now see if the file actually
+               exists, possibly under an alias.  */
+            if (matched) {
+              string found = NULL;
+              if (kpathsea_readable_file (kpse, db_file)) {
+                found = db_file;
+
+              } else {
+                const_string *a;
+
+                free (db_file); /* `db_file' wasn't on disk.  */
+
+                /* The hit in the DB doesn't exist in disk.  Now try all its
+                   aliases.  For example, suppose we have a hierarchy on CD,
+                   thus `mf.bas', but ls-R contains `mf.base'.  Find it anyway.
+                   Could probably work around this with aliases, but
+                   this is pretty easy and shouldn't hurt.  The upshot is that
+                   if one of the aliases actually exists, we use that.  */
+                for (a = aliases + 1; *a && !found; a++) {
+                  string atry = concat (*db_dirs, *a);
+                  if (kpathsea_readable_file (kpse, atry))
+                    found = atry;
+                  else
+                    free (atry);
+                }
               }
-              
-              /* On to the next directory, if any.  */
-              db_dirs++;
+
+              /* If we have a real file, add it to the list, maybe done.  */
+              if (found) {
+                str_list_add (ret, found);
+                if (!all && found)
+                  done = true;
+              }
+            } else { /* no match in the db */
+              free (db_file);
+            }
+
+            /* On to the next directory, if any.  */
+            db_dirs++;
           }
-          
+
           /* This is just the space for the pointers, not the strings.  */
           if (orig_dirs && *orig_dirs)
               free (orig_dirs);
       }
-      
-      free (aliases);
-      if (path != path_elt)
-          free((string)path);
-  }
-  return ret;
-  
-}
 
-#if defined(KPSE_COMPAT_API)
-str_list_type *
-kpse_db_search_list (const_string* names,  const_string path_elt,
-                     boolean all)
-{
-    return kpathsea_db_search_list (kpse_def, names, path_elt, all);
+      free ((void *) aliases);
+      if (temp_str)
+          free (temp_str);
+  }
+
+  return ret;
 }
-#endif
