@@ -1,6 +1,6 @@
 /* truncate.h: truncate too-long components in a filename.
 
-   Copyright 1993, 2008 Karl Berry.
+   Copyright 1993, 2008, 2010, 2011 Karl Berry.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -18,6 +18,8 @@
 #ifndef KPATHSEA_TRUNCATE_H
 #define KPATHSEA_TRUNCATE_H
 
+#ifdef MAKE_KPSE_DLL /* libkpathsea internal only */
+
 #include <kpathsea/c-proto.h>
 #include <kpathsea/types.h>
 
@@ -26,8 +28,6 @@
    NAME.  */
 extern string kpathsea_truncate_filename (kpathsea kpse, const_string name);
 
-#if defined (KPSE_COMPAT_API)
-extern string kpse_truncate_filename (const_string name);
-#endif
+#endif /* MAKE_KPSE_DLL */
 
 #endif /* not KPATHSEA_TRUNCATE_H */
