@@ -13,7 +13,7 @@
 #endif
 
 /* We must include this first, to resolve many C issues.  */
-#include "config.h"
+#include <w2c/config.h>
 
 /* We only use getopt in the applications, not in web2c itself.  */
 #include <kpathsea/getopt.h>
@@ -57,8 +57,8 @@
 
 #define floorunscaled(i) ((i)>>16)
 #define floorscaled(i) ((i)&(-65536))
-#define roundunscaled(i) (((i>>15)+1)>>1)
-#define roundfraction(i) (((i>>11)+1)>>1)
+#define roundunscaled(i) ((((i)>>15)+1)>>1)
+#define roundfraction(i) ((((i)>>11)+1)>>1)
 #ifndef TeX
 /* In TeX, the half routine is always applied to positive integers.
    In MF and MP, it isn't; therefore, we can't portably use the C shift
@@ -122,9 +122,11 @@ typedef FILE *text;
         FATAL1 ("putbyte(%ld) failed", (long) x); } while (0)
 
 /* To work around casting problems.  */
+#define ucharcast(x) ((unsigned char) (x))
 #define intcast(x) ((integer) (x))
 #define stringcast(x) ((string) (x))
 #define conststringcast(x) ((const_string) (x))
+#define ustringcast(x) ((unsigned char *) (x))
 
 /* For throwing away input from the file F.  */
 #define vgetc(f) (void) getc (f)
@@ -143,9 +145,18 @@ typedef FILE *text;
 typedef unsigned char *pointertobyte;
 #define casttobytepointer(e) ((pointertobyte) e)
 
-/* Write out elements START through END of BUF to the file F.  For gftodvi.  */
-#define writechunk(f, buf, start, end) \
-  (void) fwrite (&buf[start], sizeof (buf[start]), end - start + 1, f)
+/* How to output to the GF or DVI file.  */
+#define WRITE_OUT(a, b)							\
+  if ((size_t) fwrite ((char *) &OUT_BUF[a], sizeof (OUT_BUF[a]),       \
+                    (size_t) ((size_t)(b) - (size_t)(a) + 1), OUT_FILE) \
+      != (size_t) ((size_t) (b) - (size_t) (a) + 1))                    \
+    FATAL_PERROR ("fwrite");
+
+#ifdef GFTODVI
+#define writedvi WRITE_OUT 
+#define OUT_FILE dvifile
+#define OUT_BUF dvibuf
+#endif
 
 /* PatGen 2 uses this.  */
 #define input2ints(a,b) zinput2ints (&a, &b)
@@ -159,17 +170,28 @@ typedef unsigned char *pointertobyte;
 #define xmallocarray(type,size) ((type*)xmalloc((size+1)*sizeof(type)))
 /* Same for reallocating an array. */
 #define xreallocarray(ptr,type,size) ((type*)xrealloc(ptr,(size+1)*sizeof(type)))
+/* Allocate and clear an array of a given type. Add 1 to nmemb and size. */
+#define xcallocarray(type,nmemb,size) ((type*)xcalloc(nmemb+1,(size+1)*sizeof(type)))
 
 /* BibTeX needs this to dynamically reallocate arrays.  Too bad we can't
    rely on stringification, or we could avoid the ARRAY_NAME arg.
    Actually allocate one more than requests, so we can index the last
    entry, as Pascal wants to do.  */
+#define BIBXRETALLOCNOSET(array_name, array_var, type, size_var, new_size) \
+  fprintf (logfile, "Reallocated %s (elt_size=%ld) to %ld items from %ld.\n", \
+           array_name, (long) sizeof (type), (long) new_size, (long) size_var); \
+  XRETALLOC (array_var, new_size + 1, type)
+/* Same as above, but also increase SIZE_VAR when no more arrays
+   with the same size parameter will be resized.  */
 #define BIBXRETALLOC(array_name, array_var, type, size_var, new_size) do { \
-  fprintf (logfile, "Reallocated %s (elt_size=%d) to %ld items from %ld.\n", \
-           array_name, (int) sizeof (type), new_size, size_var); \
-  XRETALLOC (array_var, new_size + 1, type); \
+  BIBXRETALLOCNOSET(array_name, array_var, type, size_var, new_size); \
   size_var = new_size; \
 } while (0)
+/* Same as above, but for the pseudo-TYPE ASCII_code[LENGTH+1].  */
+#define BIBXRETALLOCSTRING(array_name, array_var, length, size_var, new_size) \
+  fprintf (logfile, "Reallocated %s (elt_size=%ld) to %ld items from %ld.\n", \
+           array_name, (long) (length + 1), (long) new_size, (long) size_var); \
+  XRETALLOC (array_var, (new_size) * (length + 1), ASCIIcode)
   
 /* Need precisely int for getopt, etc. */
 #define cinttype int
@@ -180,6 +202,11 @@ typedef unsigned char *pointertobyte;
 
 #define constcstring const_string
 
+/* For strings of unsigned chars, used as array indices.  */
+#define constw2custring const_w2custring
+typedef unsigned char *w2custring;
+typedef const unsigned char *const_w2custring;
+
 /* Not all C libraries have fabs, so we'll roll our own.  */
 #undef fabs
 #define fabs(x) ((x) >= 0.0 ? (x) : -(x))
@@ -225,13 +252,13 @@ typedef unsigned char *pointertobyte;
 #define kpsefindfile	kpse_find_file
 #define kpsefindmf	kpse_find_mf
 #define kpsefindmft	kpse_find_mft
-#define kpsefindofm    kpse_find_ofm
-#define kpsefindovf    kpse_find_ovf
+#define kpsefindofm	kpse_find_ofm
+#define kpsefindovf	kpse_find_ovf
 #define kpsefindtex	kpse_find_tex
 #define kpsefindtfm	kpse_find_tfm
 #define kpsefindvf	kpse_find_vf
+#define kpseinnameok	kpse_in_name_ok
 #define kpseinitprog	kpse_init_prog
-#define kpsesetprogname kpse_set_progname
 #define kpsesetprogramname kpse_set_program_name
 #define kpseresetprogramname kpse_reset_program_name
 #define kpsegfformat	kpse_gf_format
@@ -244,6 +271,7 @@ typedef unsigned char *pointertobyte;
 #define kpseofmformat	kpse_ofm_format
 #define kpseoplformat	kpse_opl_format
 #define kpseotpformat	kpse_otp_format
+#define kpseoutnameok	kpse_out_name_ok
 #define kpseovpformat	kpse_ovp_format
 #define kpseovfformat	kpse_ovf_format
 #define kpseopenfile	kpse_open_file
@@ -254,10 +282,9 @@ typedef unsigned char *pointertobyte;
 #define kpsevarvalue	kpse_var_value
 #define kpsesetprogramenabled	kpse_set_program_enabled
 #define kpsesrccmdline	kpse_src_cmdline
-#define makesuffix	make_suffix
+#define kpsesrccompile	kpse_src_compile
 #define recorderchangefilename	recorder_change_filename
 #define recorderenabled	recorder_enabled
-#define removesuffix	remove_suffix
 
 /* We need a new type for the argument parsing, too.  */
 typedef struct option getoptstruct;
