@@ -1,6 +1,6 @@
 /* limglib.c
    
-   Copyright 2006-2009 Taco Hoekwater <taco@luatex.org>
+   Copyright 2006-2010 Taco Hoekwater <taco@luatex.org>
 
    This file is part of LuaTeX.
 
@@ -18,16 +18,16 @@
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
 static const char _svn_version[] =
-    "$Id: limglib.c 3261 2009-12-18 11:38:21Z taco $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.50.0/source/texk/web2c/luatexdir/lua/limglib.c $";
+    "$Id: limglib.c 4051 2011-01-09 22:41:33Z hhenkel $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.66.0/source/texk/web2c/luatexdir/lua/limglib.c $";
 
+#include "ptexlib.h"
+#include "lua/luatex-api.h"
 #include <stdio.h>
 #include <string.h>
 #include <assert.h>
-#include <../lua51/lua.h>
-#include <../lua51/lauxlib.h>
-#include <ptexlib.h>
-#include "lua/luatex-api.h"
+#include "lua51/lua.h"
+#include "lua51/lauxlib.h"
 
 /**********************************************************************/
 
@@ -47,7 +47,7 @@ void stackDump(lua_State * L, char *s)
             printf(lua_toboolean(L, i) ? "true" : "false");
             break;
         case LUA_TNUMBER:      /* numbers */
-            printf("%g", lua_tonumber(L, i));
+            printf("%g", (double) lua_tonumber(L, i));
             break;
         default:               /* other values */
             printf("%s", lua_typename(L, t));
@@ -67,12 +67,7 @@ typedef enum { P__ZERO, P_ATTR, P_BBOX, P_COLORDEPTH, P_COLORSPACE, P_DEPTH,
     P_WIDTH, P_XRES, P_XSIZE, P_YRES, P_YSIZE, P__SENTINEL
 } parm_idx;
 
-typedef struct {
-    const char *name;           /* parameter name */
-    parm_idx idx;               /* index within img_parms array */
-} parm_struct;
-
-const parm_struct img_parms[] = {
+static const parm_struct img_parms[] = {
     {NULL, P__ZERO},            /* dummy; lua indices run from 1 */
     {"attr", P_ATTR},
     {"bbox", P_BBOX},
@@ -99,9 +94,9 @@ const parm_struct img_parms[] = {
     {NULL, P__SENTINEL}
 };
 
-#define imgtype_max 5
+#define imgtype_max 6
 const char *imgtype_s[] =
-    { "none", "pdf", "png", "jpg", "jbig2", "stream", NULL };
+    { "none", "pdf", "png", "jpg", "jp2", "jbig2", "stream", NULL };
 
 #define pagebox_max 5
 const char *pdfboxspec_s[] =
@@ -119,7 +114,7 @@ static void image_to_lua(lua_State * L, image * a)
     if (!lua_isnumber(L, -1))   /* !i k u ... */
         luaL_error(L, "image_to_lua(): %s is not a valid image key",
                    lua_tostring(L, -2));
-    i = lua_tointeger(L, -1);   /* i k u ... */
+    i = (int) lua_tointeger(L, -1);     /* i k u ... */
     lua_pop(L, 2);              /* u ... */
     switch (i) {
     case P_WIDTH:
@@ -280,16 +275,16 @@ static void lua_to_image(lua_State * L, image * a)
     if (!lua_isnumber(L, -1))   /* !i v k t ... */
         luaL_error(L, "lua_to_image(): %s is not a valid image key",
                    lua_tostring(L, -3));
-    i = lua_tointeger(L, -1);   /* i v k t ... */
+    i = (int) lua_tointeger(L, -1);     /* i v k t ... */
     lua_pop(L, 1);              /* v k t ... */
     switch (i) {
     case P_WIDTH:
         if (lua_isnil(L, -1))
             set_wd_running(a);
         else if (lua_type(L, -1) == LUA_TNUMBER)
-            img_width(a) = lua_tointeger(L, -1);
+            img_width(a) = (int) lua_tointeger(L, -1);
         else if (lua_type(L, -1) == LUA_TSTRING)
-            img_width(a) = dimen_to_number(L, (char *) lua_tostring(L, -1));
+            img_width(a) = dimen_to_number(L, lua_tostring(L, -1));
         else
             luaL_error(L,
                        "image.width needs integer or nil value or dimension string");
@@ -298,9 +293,9 @@ static void lua_to_image(lua_State * L, image * a)
         if (lua_isnil(L, -1))
             set_ht_running(a);
         else if (lua_type(L, -1) == LUA_TNUMBER)
-            img_height(a) = lua_tointeger(L, -1);
+            img_height(a) = (int) lua_tointeger(L, -1);
         else if (lua_type(L, -1) == LUA_TSTRING)
-            img_height(a) = dimen_to_number(L, (char *) lua_tostring(L, -1));
+            img_height(a) = dimen_to_number(L, lua_tostring(L, -1));
         else
             luaL_error(L,
                        "image.height needs integer or nil value or dimension string");
@@ -309,16 +304,16 @@ static void lua_to_image(lua_State * L, image * a)
         if (lua_isnil(L, -1))
             set_dp_running(a);
         else if (lua_type(L, -1) == LUA_TNUMBER)
-            img_depth(a) = lua_tointeger(L, -1);
+            img_depth(a) = (int) lua_tointeger(L, -1);
         else if (lua_type(L, -1) == LUA_TSTRING)
-            img_depth(a) = dimen_to_number(L, (char *) lua_tostring(L, -1));
+            img_depth(a) = dimen_to_number(L, lua_tostring(L, -1));
         else
             luaL_error(L,
                        "image.depth needs integer or nil value or dimension string");
         break;
     case P_TRANSFORM:
         if (lua_isnumber(L, -1))
-            img_transform(a) = lua_tointeger(L, -1);
+            img_transform(a) = (int) lua_tointeger(L, -1);
         else
             luaL_error(L, "image.transform needs integer value");
         break;
@@ -329,8 +324,7 @@ static void lua_to_image(lua_State * L, image * a)
         if (img_type(d) == IMG_TYPE_PDFSTREAM)
             luaL_error(L, "image.filename can't be used with image.stream");
         if (lua_isstring(L, -1)) {
-            if (img_filename(d) != NULL)
-                xfree(img_filename(d));
+            xfree(img_filename(d));
             img_filename(d) = xstrdup(lua_tostring(L, -1));
         } else
             luaL_error(L, "image.filename needs string value");
@@ -339,8 +333,7 @@ static void lua_to_image(lua_State * L, image * a)
         if (img_state(d) >= DICT_FILESCANNED)
             luaL_error(L, "image.attr is now read-only");
         if (lua_isstring(L, -1) || lua_isnil(L, -1)) {
-            if (img_attr(d) != NULL)
-                xfree(img_attr(d));
+            xfree(img_attr(d));
             if (lua_isstring(L, -1))
                 img_attr(d) = xstrdup(lua_tostring(L, -1));
         } else
@@ -350,15 +343,12 @@ static void lua_to_image(lua_State * L, image * a)
         if (img_state(d) >= DICT_FILESCANNED)
             luaL_error(L, "image.page is now read-only");
         if (lua_type(L, -1) == LUA_TSTRING) {
-            if (img_pagename(d) != NULL)
-                xfree(img_pagename(d));
+            xfree(img_pagename(d));
             img_pagename(d) = xstrdup(lua_tostring(L, -1));
             img_pagenum(d) = 0;
         } else if (lua_type(L, -1) == LUA_TNUMBER) {
-            img_pagenum(d) = lua_tointeger(L, -1);
-            if (img_pagename(d) != NULL)
-                xfree(img_pagename(d));
-            img_pagename(d) = NULL;
+            img_pagenum(d) = (int) lua_tointeger(L, -1);
+            xfree(img_pagename(d));
         } else
             luaL_error(L, "image.page needs integer or string value");
         break;
@@ -368,7 +358,7 @@ static void lua_to_image(lua_State * L, image * a)
         if (lua_isnil(L, -1))
             img_colorspace(d) = 0;
         else if (lua_isnumber(L, -1))
-            img_colorspace(d) = lua_tointeger(L, -1);
+            img_colorspace(d) = (int) lua_tointeger(L, -1);
         else
             luaL_error(L, "image.colorspace needs integer or nil value");
         break;
@@ -393,10 +383,9 @@ static void lua_to_image(lua_State * L, image * a)
             lua_pushinteger(L, i);      /* idx v k t ... */
             lua_gettable(L, -2);        /* int v k t ... */
             if (lua_type(L, -1) == LUA_TNUMBER)
-                img_bbox(d)[i - 1] = lua_tointeger(L, -1);
+                img_bbox(d)[i - 1] = (int) lua_tointeger(L, -1);
             else if (lua_type(L, -1) == LUA_TSTRING)
-                img_bbox(d)[i - 1] =
-                    dimen_to_number(L, (char *) lua_tostring(L, -1));
+                img_bbox(d)[i - 1] = dimen_to_number(L, lua_tostring(L, -1));
             else
                 luaL_error(L,
                            "image.bbox table needs integer value or dimension string elements");
@@ -411,8 +400,7 @@ static void lua_to_image(lua_State * L, image * a)
             luaL_error(L, "image.stream is now read-only");
         if (img_pdfstream_ptr(d) == NULL)
             new_img_pdfstream_struct(d);
-        if (img_pdfstream_stream(d) != NULL)
-            xfree(img_pdfstream_stream(d));
+        xfree(img_pdfstream_stream(d));
         img_pdfstream_stream(d) = xstrdup(lua_tostring(L, -1));
         img_type(d) = IMG_TYPE_PDFSTREAM;
         break;
@@ -436,7 +424,7 @@ static void lua_to_image(lua_State * L, image * a)
 
 /**********************************************************************/
 
-void copy_image(lua_State * L, lua_Number scale)
+static void copy_image(lua_State * L, lua_Number scale)
 {
     image *a, **aa, *b, **bb;
     if (lua_gettop(L) != 1)
@@ -534,7 +522,7 @@ static int l_scan_image(lua_State * L)
     aa = (image **) luaL_checkudata(L, 1, TYPE_IMG);    /* image */
     a = *aa;
     check_o_mode(static_pdf, "img.scan()", 1 << OMODE_PDF, false);
-    /* flush_str(last_tex_string); */ /* ?? */
+    /* flush_str(last_tex_string); *//* ?? */
     read_scale_img(a);
     return 1;                   /* image */
 }
@@ -560,20 +548,18 @@ typedef enum { WR_WRITE, WR_IMMEDIATEWRITE, WR_NODE, WR_VF_IMG } wrtype_e;
 const char *wrtype_s[] =
     { "img.write()", "img.immediatewrite()", "img.node()", "write vf image" };
 
-extern void lua_nodelib_push_fast(lua_State * L, halfword n);
-
 static void setup_image(PDF pdf, image * a, wrtype_e writetype)
 {
     image_dict *ad;
     assert(a != NULL);
     ad = img_dict(a);
-    check_o_mode(pdf, (char *) wrtype_s[writetype], 1 << OMODE_PDF, false);
-    /* flush_str(last_tex_string); */ /* ?? */
+    check_o_mode(pdf, wrtype_s[writetype], 1 << OMODE_PDF, false);
+    /* flush_str(last_tex_string); *//* ?? */
     read_scale_img(a);
     if (img_objnum(ad) == 0) {  /* latest needed just before out_img() */
         pdf->ximage_count++;
-        pdf_create_obj(pdf, obj_type_ximage, pdf->ximage_count);
-        img_objnum(ad) = pdf->obj_ptr;
+        img_objnum(ad) =
+            pdf_create_obj(pdf, obj_type_ximage, pdf->ximage_count);
         img_index(ad) = pdf->ximage_count;
         idict_to_array(ad);     /* now ad is read-only */
         obj_data_ptr(pdf, pdf->obj_ptr) = img_index(ad);
@@ -623,6 +609,8 @@ static int l_write_image(lua_State * L)
 static int l_immediatewrite_image(lua_State * L)
 {
     check_o_mode(static_pdf, "img.immediatewrite()", 1 << OMODE_PDF, true);
+    if (global_shipping_mode != NOT_SHIPPING)
+        luaL_error(L, "pdf.immediatewrite() can not be used with \\latelua");
     write_image_or_node(L, WR_IMMEDIATEWRITE);
     return 1;                   /* image */
 }
@@ -650,11 +638,11 @@ static int l_image_keys(lua_State * L)
 static int l_image_types(lua_State * L)
 {
     int i;
-    char **p;
+    const char **p;
     if (lua_gettop(L) != 0)
         luaL_error(L, "img.types() goes without argument");
     lua_newtable(L);            /* t */
-    for (i = 1, p = (char **) (imgtype_s + 1); *p != NULL; p++, i++) {
+    for (i = 1, p = (const char **) (imgtype_s + 1); *p != NULL; p++, i++) {
         lua_pushinteger(L, (int) i);    /* k t */
         lua_pushstring(L, *p);  /* v k t */
         lua_settable(L, -3);    /* t */
@@ -665,11 +653,11 @@ static int l_image_types(lua_State * L)
 static int l_image_boxes(lua_State * L)
 {
     int i;
-    char **p;
+    const char **p;
     if (lua_gettop(L) != 0)
         luaL_error(L, "img.boxes() goes without argument");
     lua_newtable(L);            /* t */
-    for (i = 1, p = (char **) (pdfboxspec_s + 1); *p != NULL; p++, i++) {
+    for (i = 1, p = (const char **) (pdfboxspec_s + 1); *p != NULL; p++, i++) {
         lua_pushinteger(L, (int) i);    /* k t */
         lua_pushstring(L, *p);  /* v k t */
         lua_settable(L, -3);    /* t */
@@ -697,7 +685,7 @@ void vf_out_image(PDF pdf, unsigned i)
     image *a, **aa;
     image_dict *ad;
     lua_State *L = Luas;        /* ... */
-    lua_rawgeti(L, LUA_GLOBALSINDEX, i);        /* image ... */
+    lua_rawgeti(L, LUA_GLOBALSINDEX, (int) i);  /* image ... */
     aa = (image **) luaL_checkudata(L, -1, TYPE_IMG);
     a = *aa;
     ad = img_dict(a);
@@ -803,19 +791,6 @@ static const struct luaL_Reg img_dict_m[] = {
 
 /**********************************************************************/
 
-void preset_environment(lua_State * L, const parm_struct * p)
-{
-    int i;
-    lua_newtable(L);            /* t */
-    for (i = 1, ++p; p->name != NULL; i++, p++) {
-        assert(i == (int) p->idx);
-        lua_pushstring(L, p->name);     /* k t */
-        lua_pushinteger(L, (int) p->idx);       /* v k t */
-        lua_settable(L, -3);    /* t */
-    }
-    lua_replace(L, LUA_ENVIRONINDEX);   /* - */
-}
-
 int luaopen_img(lua_State * L)
 {
     preset_environment(L, img_parms);
