@@ -17,11 +17,11 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
+#include "ptexlib.h"
 #include "lua/luatex-api.h"
-#include <ptexlib.h>
 
 static const char _svn_version[] =
-    "$Id: llualib.c 3261 2009-12-18 11:38:21Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.50.0/source/texk/web2c/luatexdir/lua/llualib.c $";
+    "$Id: llualib.c 3615 2010-04-13 21:59:59Z oneiros $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.66.0/source/texk/web2c/luatexdir/lua/llualib.c $";
 
 #define LOAD_BUF_SIZE 256
 #define UINT_MAX32 0xFFFFFFFF
@@ -40,8 +40,6 @@ unsigned int luabytecode_bytes = 0;
 
 char *luanames[65536] = { NULL };
 
-extern char *luanames[];
-
 char *get_lua_name(int i)
 {
     if (i < 0 || i > 65535)
@@ -74,7 +72,7 @@ void dump_luac_registers(void)
     for (k = 0; k < 65536; k++) {
         char *a = luanames[k];
         if (a != NULL) {
-            x = strlen(a) + 1;
+            x = (int) strlen(a) + 1;
             dump_int(x);
             dump_things(*a, x);
         } else {
@@ -92,12 +90,12 @@ void undump_luac_registers(void)
     bytecode b;
     undump_int(luabytecode_max);
     if (luabytecode_max >= 0) {
-        i = (luabytecode_max + 1);
-        if ((int) (UINT_MAX32 / sizeof(bytecode) + 1) <= i) {
+        i = (unsigned) (luabytecode_max + 1);
+        if ((int) (UINT_MAX32 / (int) sizeof(bytecode) + 1) <= i) {
             lua_fatal_error("Corrupt format file");
         }
-        lua_bytecode_registers = xmalloc(i * sizeof(bytecode));
-        luabytecode_bytes = i * sizeof(bytecode);
+        lua_bytecode_registers = xmalloc((unsigned) (i * sizeof(bytecode)));
+        luabytecode_bytes = (unsigned) (i * sizeof(bytecode));
         for (i = 0; i <= (unsigned) luabytecode_max; i++) {
             lua_bytecode_registers[i].done = 0;
             lua_bytecode_registers[i].size = 0;
@@ -108,9 +106,9 @@ void undump_luac_registers(void)
             undump_int(k);
             undump_int(x);
             b.size = x;
-            b.buf = xmalloc(b.size);
-            luabytecode_bytes += b.size;
-            memset(b.buf, 0, b.size);
+            b.buf = xmalloc((unsigned) b.size);
+            luabytecode_bytes += (unsigned) b.size;
+            memset(b.buf, 0, (size_t) b.size);
             do_zundump((char *) b.buf, 1, b.size, DUMP_FILE);
             lua_bytecode_registers[k].size = b.size;
             lua_bytecode_registers[k].alloc = b.size;
@@ -120,7 +118,7 @@ void undump_luac_registers(void)
     for (k = 0; k < 65536; k++) {
         undump_int(x);
         if (x > 0) {
-            char *s = xmalloc(x);
+            char *s = xmalloc((unsigned) x);
             undump_things(*s, x);
             luanames[k] = s;
         }
@@ -159,21 +157,23 @@ static int bytecode_register_shadow_get(lua_State * L, int k)
 }
 
 
-int writer(lua_State * L, const void *b, size_t size, void *B)
+static int writer(lua_State * L, const void *b, size_t size, void *B)
 {
     bytecode *buf = (bytecode *) B;
     (void) L;                   /* for -Wunused */
-    if ((int) (buf->size + size) > buf->alloc) {
-        buf->buf = xrealloc(buf->buf, buf->alloc + size + LOAD_BUF_SIZE);
-        buf->alloc = buf->alloc + size + LOAD_BUF_SIZE;
+    if ((int) (buf->size + (int) size) > buf->alloc) {
+        buf->buf =
+            xrealloc(buf->buf,
+                     (unsigned) (buf->alloc + (int) size + LOAD_BUF_SIZE));
+        buf->alloc = buf->alloc + (int) size + LOAD_BUF_SIZE;
     }
     memcpy(buf->buf + buf->size, b, size);
-    buf->size += size;
-    luabytecode_bytes += size;
+    buf->size += (int) size;
+    luabytecode_bytes += (unsigned) size;
     return 0;
 }
 
-const char *reader(lua_State * L, void *ud, size_t * size)
+static const char *reader(lua_State * L, void *ud, size_t * size)
 {
     bytecode *buf = (bytecode *) ud;
     (void) L;                   /* for -Wunused */
@@ -182,12 +182,12 @@ const char *reader(lua_State * L, void *ud, size_t * size)
         buf->done = 0;
         return NULL;
     }
-    *size = buf->size;
+    *size = (size_t) buf->size;
     buf->done = buf->size;
     return (const char *) buf->buf;
 }
 
-int get_bytecode(lua_State * L)
+static int get_bytecode(lua_State * L)
 {
     int k;
     k = (int) luaL_checkinteger(L, -1);
@@ -198,8 +198,7 @@ int get_bytecode(lua_State * L)
             if (lua_load
                 (L, reader, (void *) (lua_bytecode_registers + k),
                  "bytecode")) {
-                lua_error(L);
-                lua_pushnil(L);
+		return luaL_error(L, "bad bytecode register");
             } else {
                 lua_pushvalue(L, -1);
                 bytecode_register_shadow_set(L, k);
@@ -211,34 +210,34 @@ int get_bytecode(lua_State * L)
     return 1;
 }
 
-int set_bytecode(lua_State * L)
+static int set_bytecode(lua_State * L)
 {
     int k, ltype;
     unsigned int i;
     k = (int) luaL_checkinteger(L, -2);
-    i = k + 1;
+    i = (unsigned) k + 1;
     if ((int) (UINT_MAX32 / sizeof(bytecode) + 1) < i) {
-        lua_pushstring(L, "value too large");
-        lua_error(L);
+        luaL_error(L, "value too large");
     }
     if (k < 0) {
-        lua_pushstring(L, "negative values not allowed");
-        lua_error(L);
+        luaL_error(L, "negative values not allowed");
     }
     ltype = lua_type(L, -1);
     if (ltype != LUA_TFUNCTION && ltype != LUA_TNIL) {
-        lua_pushstring(L, "unsupported type");
-        lua_error(L);
+        luaL_error(L, "unsupported type");
     }
     if (k > luabytecode_max) {
-        i = sizeof(bytecode) * (k + 1);
+        i = (unsigned) (sizeof(bytecode) * ((unsigned) k + 1));
         lua_bytecode_registers = xrealloc(lua_bytecode_registers, i);
         if (luabytecode_max == -1) {
-            luabytecode_bytes += sizeof(bytecode) * (k + 1);
+            luabytecode_bytes +=
+                (unsigned) (sizeof(bytecode) * (unsigned) (k + 1));
         } else {
-            luabytecode_bytes += sizeof(bytecode) * (k + 1 - luabytecode_max);
+            luabytecode_bytes +=
+                (unsigned) (sizeof(bytecode) *
+                            (unsigned) (k + 1 - luabytecode_max));
         }
-        for (i = (luabytecode_max + 1); i <= (unsigned) k; i++) {
+        for (i = (unsigned) (luabytecode_max + 1); i <= (unsigned) k; i++) {
             lua_bytecode_registers[i].buf = NULL;
             lua_bytecode_registers[i].size = 0;
             lua_bytecode_registers[i].done = 0;
@@ -247,8 +246,7 @@ int set_bytecode(lua_State * L)
     }
     if (lua_bytecode_registers[k].buf != NULL) {
         xfree(lua_bytecode_registers[k].buf);
-        luabytecode_bytes -= lua_bytecode_registers[k].size;
-        lua_bytecode_registers[k].buf = NULL;
+        luabytecode_bytes -= (unsigned) lua_bytecode_registers[k].size;
         lua_bytecode_registers[k].size = 0;
         lua_bytecode_registers[k].done = 0;
         lua_pushnil(L);
@@ -265,10 +263,10 @@ int set_bytecode(lua_State * L)
 }
 
 
-int set_luaname(lua_State * L)
+static int set_luaname(lua_State * L)
 {
     int k;
-    char *s;
+    const char *s;
     if (lua_gettop(L) == 3) {
         k = (int) luaL_checkinteger(L, 2);
         if (k > 65535 || k < 0) {
@@ -279,7 +277,7 @@ int set_luaname(lua_State * L)
                 luanames[k] = NULL;
             }
             if (lua_isstring(L, 3)) {
-                s = (char *) lua_tostring(L, 3);
+                s = lua_tostring(L, 3);
                 if (s != NULL)
                     luanames[k] = xstrdup(s);
             }
@@ -288,7 +286,7 @@ int set_luaname(lua_State * L)
     return 0;
 }
 
-int get_luaname(lua_State * L)
+static int get_luaname(lua_State * L)
 {
     int k;
     k = (int) luaL_checkinteger(L, 2);
