@@ -17,17 +17,17 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
+#include "ptexlib.h"
 #include "lua/luatex-api.h"
-#include <ptexlib.h>
 
-#include <sys/stat.h>
-#include <sys/types.h>
+#include <kpathsea/c-stat.h>
+#include <kpathsea/c-dir.h>
 #include <time.h>
 
 static const char _svn_version[] =
-    "$Id: loslibext.c 3288 2009-12-24 10:05:46Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.50.0/source/texk/web2c/luatexdir/lua/loslibext.c $";
+    "$Id: loslibext.c 4167 2011-04-16 09:20:26Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/branches/0.70.x/source/texk/web2c/luatexdir/lua/loslibext.c $";
 
-#if defined(_WIN32) || defined(WIN32) || defined(__NT__)
+#if defined(_WIN32) || defined(__NT__)
 #  define MKDIR(a,b) mkdir(a)
 #else
 #  define MKDIR(a,b) mkdir(a,b)
@@ -39,7 +39,7 @@ static const char _svn_version[] =
   successfully compile without major prorting effort 
   (amiga|mac|os2|vms) */
 
-#if defined(_WIN32) || defined(WIN32) || defined(__NT__)
+#if defined(_WIN32) || defined(__NT__)
 #  define OS_PLATTYPE "windows"
 #  define OS_PLATNAME "windows"
 #elif defined(__GO32__) || defined(__DJGPP__) || defined(__DOS__)
@@ -51,6 +51,8 @@ static const char _svn_version[] =
 /* this is just a first guess */
 #  if defined(__BSD__)
 #    define OS_PLATNAME "bsd"
+#  elif defined(__CYGWIN__)
+#    define OS_PLATNAME "cygwin"
 #  elif defined(__SYSV__)
 #    define OS_PLATNAME "sysv"
 #  else
@@ -63,6 +65,9 @@ static const char _svn_version[] =
 #  elif defined(__FREEBSD__) || defined(__FreeBSD__) || defined(__FreeBSD)
 #    undef OS_PLATNAME
 #    define OS_PLATNAME "freebsd"
+#  elif defined(__FreeBSD_kernel__)
+#    undef OS_PLATNAME
+#    define OS_PLATNAME "kfreebsd"
 #  elif defined(__OPENBSD__)  || defined(__OpenBSD)
 #    undef OS_PLATNAME
 #    define OS_PLATNAME "openbsd"
@@ -96,7 +101,7 @@ static const char _svn_version[] =
  * gettimeofday() for win32 is using an alternative definition
  */
 
-#if (! defined(WIN32)) && (! defined(__SUNOS__))
+#if (! defined(_WIN32)) && (! defined(__SUNOS__))
 #  include <sys/time.h>         /* gettimeofday() */
 #  include <sys/times.h>        /* times() */
 #  include <sys/wait.h>
@@ -106,13 +111,33 @@ static const char _svn_version[] =
 
 #define DONT_REALLY_EXIT 1
 
-#ifdef WIN32
+/* Note: under WIN32, |environ| is nothing but a copy of the actual
+   environment as it was during program startup. That variable
+   can then be changed (which is a little odd), but such changes 
+   do not survive in the actual environment *unless* they are
+   passed on down in e.g. |_execvpe|, when they would be inherited
+   by the child process.  
+
+   This gives trouble because some parts of the texk code actually 
+   change the contents of the |environ| variable as a side-effect 
+   of other processing, and that explains why the current code does 
+   not use |environ| at all for the moment.
+
+   The API is kept in place for now, just in case. Thanks to Tomek
+   for the patch and the persistence in tracking this down.
+*/
+
+#ifdef _WIN32
 #  include <process.h>
-#  define spawn_command(a,b,c) _spawnvpe(_P_WAIT,(const char *)a,(const char* const*)b,(const char* const*)c)
+#  define spawn_command(a,b,c) c ? \
+  _spawnvpe(_P_WAIT,(const char *)a,(const char* const*)b,(const char* const*)c) : \
+  _spawnvp(_P_WAIT,(const char *)a,(const char* const*)b)
 #  if DONT_REALLY_EXIT
 #    define exec_command(a,b,c) exit(spawn_command((a),(b),(c)))
 #  else
-#    define exec_command(a,b,c) _execvpe((const char *)a,(const char* const*)b,(const char* const*)c)
+#    define exec_command(a,b,c) c ? \
+       _execvpe((const char *)a,(const char* const*)b,(const char* const*)c) : \
+       _execvp((const char *)a,(const char* const*)b)
 #  endif
 #else
 #  include <unistd.h>
@@ -125,7 +150,7 @@ static int exec_command(const char *file, char *const *argv, char *const *envp)
     size_t prefixlen, filelen, totallen;
 
     if (strchr(file, '/'))      /* Specific path */
-        return execve(file, argv, envp);
+        return envp ? execve(file, argv, envp) : execv(file, argv);
 
     filelen = strlen(file);
     path = NULL;
@@ -139,25 +164,25 @@ static int exec_command(const char *file, char *const *argv, char *const *envp)
     do {
         esp = strchr(searchpath, ':');
         if (esp)
-            prefixlen = esp - searchpath;
+            prefixlen = (size_t) (esp - searchpath);
         else
             prefixlen = strlen(searchpath);
 
         if (prefixlen == 0 || searchpath[prefixlen - 1] == '/') {
             totallen = prefixlen + filelen;
-#ifdef PATH_MAX
+#  ifdef PATH_MAX
             if (totallen >= PATH_MAX)
                 continue;
-#endif
+#  endif
             path = malloc(totallen + 1);
             memcpy(path, searchpath, prefixlen);
             memcpy(path + prefixlen, file, filelen);
         } else {
             totallen = prefixlen + filelen + 1;
-#ifdef PATH_MAX
+#  ifdef PATH_MAX
             if (totallen >= PATH_MAX)
                 continue;
-#endif
+#  endif
             path = malloc(totallen + 1);
             memcpy(path, searchpath, prefixlen);
             path[prefixlen] = '/';
@@ -165,7 +190,11 @@ static int exec_command(const char *file, char *const *argv, char *const *envp)
         }
         path[totallen] = '\0';
 
-        execve(path, argv, envp);
+        if (envp) {
+          execve(path, argv, envp);
+        } else {
+          execv(path, argv);
+        }
         free(path);
         path = NULL;
         if (errno == E2BIG || errno == ENOEXEC ||
@@ -254,18 +283,16 @@ static int spawn_command(const char *file, char *const *argv, char *const *envp)
 
 #endif
 
-extern char **environ;
-
-#ifdef WIN32
+#ifdef _WIN32
 static char *get_command_name(char *maincmd)
 {
     /* retrieve argv[0] part from the command string, 
-    it will be truncated to MAX_PATH if it's too long */
+       it will be truncated to MAX_PATH if it's too long */
     char *cmdname = (char *) malloc(sizeof(char) * MAX_PATH);
     int i, k, quoted;
     quoted = k = 0;
-    for (i = 0; (i < MAX_PATH) && maincmd[i] && 
-                (maincmd[i] != ' ' && maincmd[i] != '\t' || quoted); i++) {
+    for (i = 0; (i < MAX_PATH) && maincmd[i] &&
+	     ((maincmd[i] != ' ' && maincmd[i] != '\t') || quoted); i++) {
         if (maincmd[i] == '"') {
             quoted = !quoted;
         } else {
@@ -278,22 +305,22 @@ static char *get_command_name(char *maincmd)
 }
 #endif
 
-static char **do_split_command(char *maincmd, char **runcmd)
+static char **do_split_command(const char *maincmd, char **runcmd)
 {
     char **cmdline = NULL;
-#ifdef WIN32
+#ifdef _WIN32
     /* On WIN32 don't split anything, because 
-    _spawnvpe can't put it back together properly 
-    if there are quoted arguments with spaces. 
-    Instead, dump everything into one argument 
-    and it will be passed through as is */
+       _spawnvpe can't put it back together properly 
+       if there are quoted arguments with spaces. 
+       Instead, dump everything into one argument 
+       and it will be passed through as is */
     cmdline = malloc(sizeof(char *) * 2);
     cmdline[0] = xstrdup(maincmd);
     cmdline[1] = NULL;
     *runcmd = get_command_name(cmdline[0]);
 #else
     char *piece, *start_piece;
-    char *cmd;
+    const char *cmd;
     unsigned int i, j;
     int ret = 0;
     int in_string = 0;
@@ -350,8 +377,9 @@ static char **do_split_command(char *maincmd, char **runcmd)
 
 static char **do_flatten_command(lua_State * L, char **runcmd)
 {
-    unsigned int i, j;
-    char *s;
+    unsigned int i;
+    int j;
+    const char *s;
     char **cmdline = NULL;
     *runcmd = NULL;
 
@@ -365,11 +393,11 @@ static char **do_flatten_command(lua_State * L, char **runcmd)
     }
     if (j == 1)
         return NULL;
-    cmdline = malloc(sizeof(char *) * (j + 1));
-    for (i = 1; i <= j; i++) {
+    cmdline = malloc(sizeof(char *) * (unsigned) (j + 1));
+    for (i = 1; i <= (unsigned) j; i++) {
         cmdline[i] = NULL;
-        lua_rawgeti(L, -1, i);
-        if (lua_isnil(L, -1) || (s = (char *) lua_tostring(L, -1)) == NULL) {
+        lua_rawgeti(L, -1, (int) i);
+        if (lua_isnil(L, -1) || (s = lua_tostring(L, -1)) == NULL) {
             lua_pop(L, 1);
             if (i == 1) {
                 xfree(cmdline);
@@ -385,8 +413,8 @@ static char **do_flatten_command(lua_State * L, char **runcmd)
     cmdline[i] = NULL;
 
     lua_rawgeti(L, -1, 0);
-    if (lua_isnil(L, -1) || (s = (char *) lua_tostring(L, -1)) == NULL) {
-#ifdef WIN32
+    if (lua_isnil(L, -1) || (s = lua_tostring(L, -1)) == NULL) {
+#ifdef _WIN32
         *runcmd = get_command_name(cmdline[0]);
 #else
         *runcmd = cmdline[0];
@@ -403,10 +431,12 @@ static char **do_flatten_command(lua_State * L, char **runcmd)
 static int os_exec(lua_State * L)
 {
     int allow = 0;
-    char *maincmd = NULL, *runcmd = NULL;
+    const char *maincmd = NULL;
+    char *runcmd = NULL;
     char *safecmd = NULL, *cmdname = NULL;
     char **cmdline = NULL;
-
+    char **envblock = NULL;
+    
     if (lua_gettop(L) != 1) {
         lua_pushnil(L);
         lua_pushliteral(L, "invalid arguments passed");
@@ -418,7 +448,7 @@ static int os_exec(lua_State * L)
         return 2;
     }
     if (lua_type(L, 1) == LUA_TSTRING) {
-        maincmd = (char *) lua_tostring(L, 1);
+        maincmd = lua_tostring(L, 1);
         cmdline = do_split_command(maincmd, &runcmd);
     } else if (lua_type(L, 1) == LUA_TTABLE) {
         cmdline = do_flatten_command(L, &runcmd);
@@ -429,24 +459,26 @@ static int os_exec(lua_State * L)
      * but I am not so eager to attempt to fix that. Just document
      * that os.exec() checks only the command name.
      */
-    if (restrictedshell == 0)
+    if (restrictedshell == 0) {
         allow = 1;
-    else
-        allow = shell_cmd_is_allowed(&runcmd, &safecmd, &cmdname);
+    } else {
+        const char *theruncmd = runcmd;
+        allow = shell_cmd_is_allowed(&theruncmd, &safecmd, &cmdname);
+    }
 
     if (allow > 0 && cmdline != NULL && runcmd != NULL) {
-#if defined(WIN32) && DONT_REALLY_EXIT
+#if defined(_WIN32) && DONT_REALLY_EXIT
         if (allow == 2)
-            exec_command(safecmd, cmdline, environ);
+            exec_command(safecmd, cmdline, envblock);
         else
-            exec_command(runcmd, cmdline, environ);
+            exec_command(runcmd, cmdline, envblock);
 #else
         {
             int r;
             if (allow == 2)
-                r = exec_command(safecmd, cmdline, environ);
+                r = exec_command(safecmd, cmdline, envblock);
             else
-                r = exec_command(runcmd, cmdline, environ);
+                r = exec_command(runcmd, cmdline, envblock);
             if (r == -1) {
                 lua_pushnil(L);
                 lua_pushfstring(L, "%s: %s", runcmd, strerror(errno));
@@ -480,9 +512,11 @@ static int os_exec(lua_State * L)
 static int os_spawn(lua_State * L)
 {
     int allow = 0;
-    char *maincmd = NULL, *runcmd = NULL;
+    const char *maincmd = NULL;
+    char *runcmd = NULL;
     char *safecmd = NULL, *cmdname = NULL;
     char **cmdline = NULL;
+    char **envblock = NULL;
     int i;
 
     if (lua_gettop(L) != 1) {
@@ -496,7 +530,7 @@ static int os_spawn(lua_State * L)
         return 2;
     }
     if (lua_type(L, 1) == LUA_TSTRING) {
-        maincmd = (char *) lua_tostring(L, 1);
+        maincmd = lua_tostring(L, 1);
         cmdline = do_split_command(maincmd, &runcmd);
     } else if (lua_type(L, 1) == LUA_TTABLE) {
         cmdline = do_flatten_command(L, &runcmd);
@@ -507,15 +541,17 @@ static int os_spawn(lua_State * L)
      * but I am not so eager to attempt to fix that. Just document
      * that os.exec() checks only the command name.
      */
-    if (restrictedshell == 0)
+    if (restrictedshell == 0) {
         allow = 1;
-    else
-        allow = shell_cmd_is_allowed(&runcmd, &safecmd, &cmdname);
+    } else {
+        const char *theruncmd = runcmd;
+        allow = shell_cmd_is_allowed(&theruncmd, &safecmd, &cmdname);
+    }
     if (allow > 0 && cmdline != NULL && runcmd != NULL) {
         if (allow == 2)
-            i = spawn_command(safecmd, cmdline, environ);
+            i = spawn_command(safecmd, cmdline, envblock);
         else
-            i = spawn_command(runcmd, cmdline, environ);
+            i = spawn_command(runcmd, cmdline, envblock);
         if (safecmd)
             free(safecmd);
         if (cmdname)
@@ -526,7 +562,7 @@ static int os_spawn(lua_State * L)
         } else if (i == -1) {
             /* this branch covers WIN32 as well as fork() and waitpid() errors */
             do_error_return(strerror(errno), errno);
-#ifndef WIN32
+#ifndef _WIN32
         } else if (i == INVALID_RET_E2BIG) {
             do_error_return(strerror(E2BIG), i);
         } else if (i == INVALID_RET_ENOENT) {
@@ -565,18 +601,19 @@ static int os_spawn(lua_State * L)
 
 static int os_setenv(lua_State * L)
 {
-    char *value, *key, *val;
-    key = (char *) luaL_optstring(L, 1, NULL);
-    val = (char *) luaL_optstring(L, 2, NULL);
+    const char *key, *val;
+    char *value;
+    key = luaL_optstring(L, 1, NULL);
+    val = luaL_optstring(L, 2, NULL);
     if (key) {
         if (val) {
-            value = xmalloc(strlen(key) + strlen(val) + 2);
+            value = xmalloc((unsigned) (strlen(key) + strlen(val) + 2));
             sprintf(value, "%s=%s", key, val);
             if (putenv(value)) {
                 return luaL_error(L, "unable to change environment");
             }
         } else {
-#if defined(WIN32) || defined(__sun__) || defined(__sun) || defined(_AIX)
+#if defined(_WIN32) || defined(__sun__) || defined(__sun) || defined(_AIX)
             value = xmalloc(strlen(key) + 2);
             sprintf(value, "%s=", key);
             if (putenv(value)) {
@@ -592,7 +629,7 @@ static int os_setenv(lua_State * L)
 }
 
 
-void find_env(lua_State * L)
+static void find_env(lua_State * L)
 {
     char *envitem, *envitem_orig;
     char *envkey;
@@ -629,17 +666,17 @@ static int ex_sleep(lua_State * L)
 {
     lua_Number interval = luaL_checknumber(L, 1);
     lua_Number units = luaL_optnumber(L, 2, 1);
-#ifdef WIN32
-    Sleep(1e3 * interval / units);
+#ifdef _WIN32
+    Sleep((DWORD) (1e3 * interval / units));
 #else                           /* assumes posix or bsd */
-    usleep(1e6 * interval / units);
+    usleep((unsigned) (1e6 * interval / units));
 #endif
     return 0;
 }
 
 
 
-#ifdef WIN32
+#ifdef _WIN32
 #  define _UTSNAME_LENGTH 65
 
 /* Structure describing the system and machine.  */
@@ -800,7 +837,7 @@ static int ex_uname(lua_State * L)
 }
 
 
-#if (! defined (WIN32))  && (! defined (__SUNOS__))
+#if (! defined (_WIN32))  && (! defined (__SUNOS__))
 static int os_times(lua_State * L)
 {
     struct tms r;
@@ -837,7 +874,7 @@ static int os_times(lua_State * L)
 static int os_gettimeofday(lua_State * L)
 {
     double v;
-#  ifndef WIN32
+#  ifndef _WIN32
     struct timeval tv;
     gettimeofday(&tv, NULL);
     v = (double) tv.tv_sec + (double) tv.tv_usec / 1000000.0;
@@ -861,18 +898,19 @@ static int os_gettimeofday(lua_State * L)
 
 static const char repl[] = "0123456789abcdefghijklmnopqrstuvwxyz";
 
-static int dirs_made = 0;
-
 #define MAXTRIES 36*36*36
 
-char *do_mkdtemp(char *tmpl)
+#ifndef HAVE_MKDTEMP
+static int dirs_made = 0;
+
+static char *do_mkdtemp(char *tmpl)
 {
     int count;
     int value;
     char *xes = &tmpl[strlen(tmpl) - 6];
     /* this is not really all that random, but it will do */
     if (dirs_made == 0) {
-        srand(time(NULL));
+        srand((unsigned) time(NULL));
     }
     value = rand();
     for (count = 0; count < MAXTRIES; value += 8413, ++count) {
@@ -897,11 +935,12 @@ char *do_mkdtemp(char *tmpl)
     }
     return NULL;
 }
+#endif
 
 static int os_tmpdir(lua_State * L)
 {
     char *s, *tempdir;
-    char *tmp = (char *) luaL_optstring(L, 1, "luatex.XXXXXX");
+    const char *tmp = luaL_optstring(L, 1, "luatex.XXXXXX");
     if (tmp == NULL ||
         strlen(tmp) < 6 || (strcmp(tmp + strlen(tmp) - 6, "XXXXXX") != 0)) {
         lua_pushnil(L);
@@ -933,8 +972,18 @@ static int os_execute(lua_State * L)
     int ret = 1;
     char *safecmd = NULL;
     char *cmdname = NULL;
-    char *cmd = (char *) luaL_optstring(L, 1, NULL);
+    const char *cmd = luaL_optstring(L, 1, NULL);
 
+    if (cmd == NULL) { /* pretend we are \.{\\pdfshellescape} */
+	if (shellenabledp <= 0) {
+	    lua_pushinteger(L, 0);
+        } else if (restrictedshell == 0) {
+	    lua_pushinteger(L, 1);
+	} else {
+	    lua_pushinteger(L, 2);
+	}
+	return 1;
+    }
     if (shellenabledp <= 0) {
         lua_pushnil(L);
         lua_pushstring(L, "All command execution disabled.");
@@ -981,7 +1030,7 @@ void open_oslibext(lua_State * L, int safer_option)
     lua_setfield(L, -2, "name");
     lua_pushcfunction(L, ex_uname);
     lua_setfield(L, -2, "uname");
-#if (! defined (WIN32))  && (! defined (__SUNOS__))
+#if (! defined (_WIN32))  && (! defined (__SUNOS__))
     lua_pushcfunction(L, os_times);
     lua_setfield(L, -2, "times");
 #endif
