@@ -1,6 +1,6 @@
 /* ltexiolib.c
    
-   Copyright 2006-2008 Taco Hoekwater <taco@luatex.org>
+   Copyright 2006-2010 Taco Hoekwater <taco@luatex.org>
 
    This file is part of LuaTeX.
 
@@ -17,13 +17,13 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
+#include "ptexlib.h"
 #include "lua/luatex-api.h"
-#include <ptexlib.h>
 
 static const char _svn_version[] =
-    "$Id: ltexiolib.c 3217 2009-12-03 16:52:01Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.50.0/source/texk/web2c/luatexdir/lua/ltexiolib.c $";
+    "$Id: ltexiolib.c 3550 2010-03-26 14:37:09Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.66.0/source/texk/web2c/luatexdir/lua/ltexiolib.c $";
 
-typedef void (*texio_printer) (char *);
+typedef void (*texio_printer) (const char *);
 
 static char *loggable_info = NULL;
 
@@ -31,7 +31,7 @@ static boolean get_selector_value(lua_State * L, int i, int *l)
 {
     boolean r = false;
     if (lua_isstring(L, i)) {
-        char *s = (char *) lua_tostring(L, i);
+        const char *s = lua_tostring(L, i);
         if (strcmp(s, "log") == 0) {
             *l = log_only;
             r = true;
@@ -43,21 +43,19 @@ static boolean get_selector_value(lua_State * L, int i, int *l)
             r = true;
         }
     } else {
-        lua_pushstring(L, "first argument is not a string");
-        lua_error(L);
+        luaL_error(L, "first argument is not a string");
     }
     return r;
 }
 
 static int do_texio_print(lua_State * L, texio_printer printfunction)
 {
-    char *s;
+    const char *s;
     int i = 1;
     int save_selector = selector;
     int n = lua_gettop(L);
     if (n == 0 || !lua_isstring(L, -1)) {
-        lua_pushstring(L, "no string to print");
-        lua_error(L);
+	luaL_error(L, "no string to print");
     }
     if (n > 1) {
         if (get_selector_value(L, i, &selector))
@@ -69,20 +67,19 @@ static int do_texio_print(lua_State * L, texio_printer printfunction)
     }
     for (; i <= n; i++) {
         if (lua_isstring(L, i)) {
-            s = (char *) lua_tostring(L, i);
+            s = lua_tostring(L, i);
             printfunction(s);
         } else {
-            lua_pushstring(L, "argument is not a string");
-            lua_error(L);
+            luaL_error(L, "argument is not a string");
         }
     }
     selector = save_selector;
     return 0;
 }
 
-static void do_texio_ini_print(lua_State * L, char *extra)
+static void do_texio_ini_print(lua_State * L, const char *extra)
 {
-    char *s;
+    const char *s;
     int i = 1;
     int l = term_and_log;
     int n = lua_gettop(L);
@@ -92,7 +89,7 @@ static void do_texio_ini_print(lua_State * L, char *extra)
     }
     for (; i <= n; i++) {
         if (lua_isstring(L, i)) {
-            s = (char *) lua_tostring(L, i);
+            s = lua_tostring(L, i);
             if (l == term_and_log || l == term_only)
                 fprintf(stdout, "%s%s", extra, s);
             if (l == log_only || l == term_and_log) {
