@@ -10,27 +10,32 @@
    #defines TeX or MF, which avoids the need for a special
    Makefile rule.  */
 #include "luatex.h"
+#include "ptexlib.h"
+#include "lua/luatex-api.h"
+#include "luatex_svnversion.h"
 
-static const char __svn_version[] =
-    "$Id: luatex.c 3295 2009-12-24 14:43:31Z taco $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.50.0/source/texk/web2c/luatexdir/luatex.c $";
+static const char _svn_version[] =
+    "$Id: luatex.c 4276 2011-05-19 05:11:57Z taco $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.70.1/source/texk/web2c/luatexdir/luatex.c $";
 
 #define TeX
 
-int luatex_version = 50;        /* \.{\\luatexversion}  */
-int luatex_revision = '0';      /* \.{\\luatexrevision}  */
+int luatex_svn = luatex_svn_revision;
+int luatex_version = 70;        /* \.{\\luatexversion}  */
+int luatex_revision = '2';      /* \.{\\luatexrevision}  */
 int luatex_date_info = -extra_version_info;     /* the compile date is negated */
-char *luatex_version_string = "beta-0.50.0";
-char *engine_name = "luatex";   /* the name of this engine */
-
-
+const char *luatex_version_string = "beta-0.70.2";
+const char *engine_name = "luatex";     /* the name of this engine */
 
 #include <kpathsea/c-ctype.h>
 #include <kpathsea/line.h>
 #include <kpathsea/readable.h>
 #include <kpathsea/variable.h>
 #include <kpathsea/absolute.h>
-#include <kpathsea/recorder.h>
+#ifdef WIN32
+#include <kpathsea/concatn.h>
+#endif
+
 
 #include <time.h>               /* For `struct tm'.  */
 #if defined (HAVE_SYS_TIME_H)
@@ -45,7 +50,6 @@ char *engine_name = "luatex";   /* the name of this engine */
 
 #include <signal.h>             /* Catch interrupts.  */
 
-#include <texmfmp-help.h>
 
 /* {tex,mf}d.h defines TeX, MF, INI, and other such symbols.
    Unfortunately there's no way to get the banner into this code, so
@@ -105,7 +109,7 @@ void mk_shellcmdlist(char *v)
 {
     char **p;
     char *q, *r;
-    int n;
+    unsigned int n;
 
     q = v;
     n = 0;
@@ -120,12 +124,12 @@ void mk_shellcmdlist(char *v)
     }
     if (*q)
         n++;
-    cmdlist = (char **) xmalloc((n + 1) * sizeof(char *));
+    cmdlist = (char **) xmalloc((unsigned) ((n + 1) * sizeof(char *)));
     p = cmdlist;
     q = v;
     while ((r = strchr(q, ',')) != 0) {
         *r = '\0';
-        *p = (char *) xmalloc(strlen(q) + 1);
+        *p = (char *) xmalloc((unsigned) strlen(q) + 1);
         strcpy(*p, q);
         *r = ',';
         r++;
@@ -133,7 +137,7 @@ void mk_shellcmdlist(char *v)
         p++;
     }
     if (*q) {
-        *p = (char *) xmalloc(strlen(q) + 1);
+        *p = (char *) xmalloc((unsigned) strlen(q) + 1);
         strcpy(*p, q);
         p++;
         *p = NULL;
@@ -204,17 +208,19 @@ static int Isspace(char c)
    should get executed.  And we set CMDNAME to its first word; this is
    what is checked against the shell_escape_commands list.  */
 
-int shell_cmd_is_allowed(char **cmd, char **safecmd, char **cmdname)
+int shell_cmd_is_allowed(const char **cmd, char **safecmd, char **cmdname)
 {
     char **p;
     char *buf;
     char *s, *d;
-    int pre, spaces;
+    const char *ss;
+    int pre;
+    unsigned spaces;
     int allow = 0;
 
     /* pre == 1 means that the previous character is a white space
        pre == 0 means that the previous character is not a white space */
-    buf = (char *) xmalloc(strlen(*cmd) + 1);
+    buf = (char *) xmalloc((unsigned) strlen(*cmd) + 1);
     strcpy(buf, *cmd);
     s = buf;
     while (Isspace(*s))
@@ -247,36 +253,37 @@ int shell_cmd_is_allowed(char **cmd, char **safecmd, char **cmdname)
     }
     if (allow == 2) {
         spaces = 0;
-        for (s = *cmd; *s; s++) {
-            if (Isspace(*s))
+        for (ss = *cmd; *ss; ss++) {
+            if (Isspace(*ss))
                 spaces++;
         }
 
         /* allocate enough memory (too much?) */
 #  ifdef WIN32
-        *safecmd = (char *) xmalloc(2 * strlen(*cmd) + 3 + 2 * spaces);
+        *safecmd =
+            (char *) xmalloc(2 * (unsigned) strlen(*cmd) + 3 + 2 * spaces);
 #  else
-        *safecmd = (char *) xmalloc(strlen(*cmd) + 3 + 2 * spaces);
+        *safecmd = (char *) xmalloc((unsigned) strlen(*cmd) + 3 + 2 * spaces);
 #  endif
 
         /* make a safe command line *safecmd */
-        s = *cmd;
-        while (Isspace(*s))
-            s++;
+        ss = *cmd;
+        while (Isspace(*ss))
+            ss++;
         d = *safecmd;
-        while (!Isspace(*s) && *s)
-            *d++ = *s++;
+        while (!Isspace(*ss) && *ss)
+            *d++ = *ss++;
 
         pre = 1;
-        while (*s) {
+        while (*ss) {
             /* Quotation given by a user.  " should always be used; we
                transform it below.  On Unix, if ' is used, simply immediately
                return a quotation error.  */
-            if (*s == '\'') {
+            if (*ss == '\'') {
                 return -1;
             }
 
-            if (*s == '"') {
+            if (*ss == '"') {
                 /* All arguments are quoted as 'foo' (Unix) or "foo" (Windows)
                    before calling system(). Therefore closing QUOTE is necessary
                    if the previous character is not a white space.
@@ -291,51 +298,51 @@ int shell_cmd_is_allowed(char **cmd, char **safecmd, char **cmdname)
                 pre = 0;
                 /* output the quotation mark for the quoted argument */
                 *d++ = QUOTE;
-                s++;
+                ss++;
 
-                while (*s != '"') {
-                    /* Closing quotation mark is missing */
-                    if (*s == '\0')
+                while (*ss != '"') {
+                    /* Illegal use of ', or closing quotation mark is missing */
+                    if (*ss == '\'' || *ss == '\0')
                         return -1;
 #  ifdef WIN32
-                    if (char_needs_quote(*s))
+                    if (char_needs_quote(*ss))
                         *d++ = '^';
 #  endif
-                    *d++ = *s++;
+                    *d++ = *ss++;
                 }
 
                 /* Closing quotation mark will be output afterwards, so
                    we do nothing here */
-                s++;
+                ss++;
 
                 /* The character after the closing quotation mark
                    should be a white space or NULL */
-                if (!Isspace(*s) && *s)
+                if (!Isspace(*ss) && *ss)
                     return -1;
 
                 /* Beginning of a usual argument */
-            } else if (pre == 1 && !Isspace(*s)) {
+            } else if (pre == 1 && !Isspace(*ss)) {
                 pre = 0;
                 *d++ = QUOTE;
 #  ifdef WIN32
-                if (char_needs_quote(*s))
+                if (char_needs_quote(*ss))
                     *d++ = '^';
 #  endif
-                *d++ = *s++;
+                *d++ = *ss++;
                 /* Ending of a usual argument */
 
-            } else if (pre == 0 && Isspace(*s)) {
+            } else if (pre == 0 && Isspace(*ss)) {
                 pre = 1;
                 /* Closing quotation mark */
                 *d++ = QUOTE;
-                *d++ = *s++;
+                *d++ = *ss++;
             } else {
                 /* Copy a character from *cmd to *safecmd. */
 #  ifdef WIN32
-                if (char_needs_quote(*s))
+                if (char_needs_quote(*ss))
                     *d++ = '^';
 #  endif
-                *d++ = *s++;
+                *d++ = *ss++;
             }
         }
         /* End of the command line */
@@ -343,6 +350,36 @@ int shell_cmd_is_allowed(char **cmd, char **safecmd, char **cmdname)
             *d++ = QUOTE;
         }
         *d = '\0';
+#ifdef WIN32
+        {
+          char *p, *q, *r;
+          p = *safecmd;
+          if (!(IS_DIR_SEP (p[0]) && IS_DIR_SEP (p[1])) &&
+              !(p[1] == ':' && IS_DIR_SEP (p[2]))) { 
+            p = (char *) kpse_var_value ("SELFAUTOLOC");
+            if (p) {
+              r = *safecmd;
+              while (*r && !Isspace(*r))
+                r++;
+              if (*r == '\0')
+                q = concatn ("\"", p, "/", *safecmd, "\"", NULL);
+              else {
+                *r = '\0';
+                r++;
+                while (*r && Isspace(*r))
+                  r++;
+                if (*r)
+                  q = concatn ("\"", p, "/", *safecmd, "\" ", r, NULL);
+                else
+                  q = concatn ("\"", p, "/", *safecmd, "\"", NULL);
+              }
+              free (p);
+              free (*safecmd);
+              *safecmd = q;
+            }
+          }
+        }
+#endif
     }
 
     return allow;
@@ -367,10 +404,12 @@ int runsystem(char *cmd)
     }
 
     /* If restrictedshell == 0, any command is allowed. */
-    if (restrictedshell == 0)
+    if (restrictedshell == 0) {
         allow = 1;
-    else
-        allow = shell_cmd_is_allowed(&cmd, &safecmd, &cmdname);
+    } else {
+        const char *thecmd = cmd;
+        allow = shell_cmd_is_allowed(&thecmd, &safecmd, &cmdname);
+    }
 
     if (allow == 1)
         (void) system(cmd);
@@ -387,27 +426,18 @@ int runsystem(char *cmd)
 
 #endif
 
-/* The main program, etc.  */
-
-extern void lua_initialize(int ac, char **av);
 
 /* What we were invoked as and with.  */
 char **argv;
 int argc;
 
 /* The C version of what might wind up in |TEX_format_default|.  */
-const_string dump_name;
+string dump_name;
 
 /* The C version of the jobname, if given. */
 const_string c_job_name;
 
-/* Full source file name. */
-extern string fullnameoffile;
-
-/* The main body of the WEB is transformed into this procedure.  */
-extern TEXDLL void mainbody(void);
-
-char *ptexbanner;
+const char *ptexbanner;
 
 #if !defined(WIN32) || defined(__MINGW32__)
 /* The entry point: set up for reading the command line, which will
@@ -427,7 +457,7 @@ int main(int ac, string * av)
     lua_initialize(ac, av);
 
     /* Call the real main program.  */
-    mainbody();
+    main_body();
 
     return EXIT_SUCCESS;
 }
@@ -454,7 +484,7 @@ void topenin(void)
             /* Don't use strcat, since in Aleph the buffer elements aren't
                single bytes.  */
             while (*ptr) {
-                buffer[k++] = *(ptr++);
+                buffer[k++] = (packed_ASCII_code) * (ptr++);
             }
             buffer[k++] = ' ';
         }
@@ -480,6 +510,9 @@ void topenin(void)
    incrementally.  Shamim Mohamed adapted it for Web2c.  */
 #if defined (TeX) && defined (IPC)
 
+# ifdef WIN32
+#  include <winsock2.h>
+# else
 #  include <sys/socket.h>
 #  include <fcntl.h>
 #  ifndef O_NONBLOCK            /* POSIX */
@@ -493,6 +526,8 @@ what the fcntl ? cannot implement IPC without equivalent for O_NONBLOCK.
 #      endif                    /* no FNDELAY */
 #    endif                      /* no O_NDELAY */
 #  endif                        /* no O_NONBLOCK */
+# endif /* !WIN32 */
+
 #  ifndef IPC_PIPE_NAME         /* $HOME is prepended to this.  */
 #    define IPC_PIPE_NAME "/.TeXview_Pipe"
 #  endif
@@ -538,6 +573,9 @@ static int ipc_is_open(void)
 
 static void ipc_open_out(void)
 {
+#ifdef WIN32
+  u_long mode = 1;
+#endif
 #  ifdef IPC_DEBUG
     fputs("tex: Opening socket for IPC output ...\n", stderr);
 #  endif
@@ -552,8 +590,13 @@ static void ipc_open_out(void)
 
     sock = socket(PF_UNIX, SOCK_STREAM, 0);
     if (sock >= 0) {
-        if (connect(sock, ipc_addr, ipc_addr_len) != 0
-            || fcntl(sock, F_SETFL, O_NONBLOCK) < 0) {
+        if (connect(sock, ipc_addr, ipc_addr_len) != 0 ||
+#ifdef WIN32
+        ioctlsocket (sock, FIONBIO, &mode) < 0
+#else
+        fcntl (sock, F_SETFL, O_NONBLOCK) < 0
+#endif
+           ) {
             close(sock);
             sock = -1;
             return;
@@ -610,50 +653,31 @@ static void ipc_snd(int n, int is_eof, char *data)
 
 
 /* This routine notifies the server if there is an eof, or the filename
-   if a new DVI file is starting.  This is the routine called by TeX.
-   Aleph defines str_start(#) as str_start_ar[# - too_big_char], with
-   too_big_char = biggest_char + 1 = 65536 (omstr.ch).*/
-
+   if a new DVI file is starting.  This is the routine called by TeX.  */
 void ipcpage(int is_eof)
 {
     static boolean begun = false;
     unsigned len = 0;
-    string p = (string) "";
+    string p = NULL;
 
     if (!begun) {
         string name;            /* Just the filename.  */
         string cwd = xgetcwd();
 
         ipc_open_out();
-#  if !defined(Aleph)
-        len = strstart[outputfilename + 1] - strstart[outputfilename];
-#  else
-        len = strstartar[outputfilename + 1 - 65536L] -
-            strstartar[outputfilename - 65536L];
-#  endif
-        name = (string) xmalloc(len + 1);
-#  if !defined(Aleph)
-        strncpy(name, (string) & strpool[strstart[outputfilename]], len);
-#  else
-        {
-            unsigned i;
-            for (i = 0; i < len; i++)
-                name[i] = strpool[i + strstartar[outputfilename - 65536L]];
-        }
-#  endif
-        name[len] = 0;
 
         /* Have to pass whole filename to the other end, since it may have
            been started up and running as a daemon, e.g., as with the NeXT
            preview program.  */
+	name = static_pdf->file_name;
         p = concat3(cwd, DIR_SEP_STRING, name);
-        free(name);
+        free(cwd);
         len = strlen(p);
         begun = true;
     }
     ipc_snd(len, is_eof, p);
 
-    if (len > 0) {
+    if (p) {
         free(p);
     }
 }
@@ -666,7 +690,7 @@ string normalize_quotes(const_string name, const_string mesg)
     boolean quoted = false;
     boolean must_quote = (strchr(name, ' ') != NULL);
     /* Leave room for quotes and NUL. */
-    string ret = (string) xmalloc(strlen(name) + 3);
+    string ret = (string) xmalloc((unsigned) strlen(name) + 3);
     string p;
     const_string q;
     p = ret;
@@ -693,7 +717,7 @@ string normalize_quotes(const_string name, const_string mesg)
    variable `interrupt'; then they will do everything needed.  */
 #ifdef WIN32
 /* Win32 doesn't set SIGINT ... */
-BOOL WINAPI catch_interrupt(DWORD arg)
+static BOOL WINAPI catch_interrupt(DWORD arg)
 {
     switch (arg) {
     case CTRL_C_EVENT:
@@ -724,9 +748,7 @@ static RETSIGTYPE catch_interrupt(int arg)
    ``Get the first line of input and prepare to start''), this is as
    good a place as any.  */
 
-void
-get_date_and_time(int * minutes, int * day,
-                  int * month, int * year)
+void get_date_and_time(int *minutes, int *day, int *month, int *year)
 {
     time_t myclock = time((time_t *) 0);
     struct tm *tmptr = localtime(&myclock);
@@ -770,13 +792,13 @@ get_date_and_time(int * minutes, int * day,
 /*
  Getting a high resolution time.
  */
-void get_seconds_and_micros(int * seconds, int * micros)
+void get_seconds_and_micros(int *seconds, int *micros)
 {
 #if defined (HAVE_GETTIMEOFDAY)
     struct timeval tv;
     gettimeofday(&tv, NULL);
-    *seconds = tv.tv_sec;
-    *micros = tv.tv_usec;
+    *seconds = (int)tv.tv_sec;
+    *micros = (int)tv.tv_usec;
 #elif defined (HAVE_FTIME)
     struct timeb tb;
     ftime(&tb);
@@ -784,7 +806,7 @@ void get_seconds_and_micros(int * seconds, int * micros)
     *micros = tb.millitm * 1000;
 #else
     time_t myclock = time((time_t *) NULL);
-    *seconds = myclock;
+    *seconds = (int) myclock;
     *micros = 0;
 #endif
 }
@@ -792,12 +814,12 @@ void get_seconds_and_micros(int * seconds, int * micros)
 /*
   Generating a better seed numbers
   */
-int getrandomseed()
+int getrandomseed(void)
 {
 #if defined (HAVE_GETTIMEOFDAY)
     struct timeval tv;
     gettimeofday(&tv, NULL);
-    return (tv.tv_usec + 1000000 * tv.tv_usec);
+    return (int)(tv.tv_usec + 1000000 * tv.tv_usec);
 #elif defined (HAVE_FTIME)
     struct timeb tb;
     ftime(&tb);
@@ -821,7 +843,7 @@ boolean input_line(FILE * f)
     /* Recognize either LF or CR as a line terminator.  */
     last = first;
     while (last < buf_size && (i = getc(f)) != EOF && i != '\n' && i != '\r')
-        buffer[last++] = i;
+        buffer[last++] = (packed_ASCII_code) i;
 
     if (i == EOF && errno != EINTR && last == first)
         return false;
@@ -855,93 +877,6 @@ boolean input_line(FILE * f)
 }
 
 
-/* This string specifies what the `e' option does in response to an
-   error message.  */
-static const_string edit_value = EDITOR;
-
-/* This procedure originally due to sjc@s1-c.  TeX & Metafont call it when
-   the user types `e' in response to an error, invoking a text editor on
-   the erroneous source file.  FNSTART is how far into FILENAME the
-   actual filename starts; FNLENGTH is how long the filename is.  */
-
-void
-calledit(packedASCIIcode * filename,
-         poolpointer fnstart, int fnlength, int linenumber)
-{
-    char *temp, *command;
-    char c;
-    int sdone, ddone, i;
-
-    sdone = ddone = 0;
-    filename += fnstart;
-
-    /* Close any open input files, since we're going to kill the job.  */
-    for (i = 1; i <= in_open; i++)
-        xfclose(input_file[i], "inputfile");
-
-    /* Replace the default with the value of the appropriate environment
-       variable or config file value, if it's set.  */
-    temp = kpse_var_value(edit_var);
-    if (temp != NULL)
-        edit_value = temp;
-
-    /* Construct the command string.  The `11' is the maximum length an
-       integer might be.  */
-    command = (string) xmalloc(strlen(edit_value) + fnlength + 11);
-
-    /* So we can construct it as we go.  */
-    temp = command;
-
-    while ((c = *edit_value++) != 0) {
-        if (c == '%') {
-            switch (c = *edit_value++) {
-            case 'd':
-                if (ddone)
-                    FATAL("call_edit: `%%d' appears twice in editor command");
-                sprintf(temp, "%ld", (long int) linenumber);
-                while (*temp != '\0')
-                    temp++;
-                ddone = 1;
-                break;
-
-            case 's':
-                if (sdone)
-                    FATAL("call_edit: `%%s' appears twice in editor command");
-                for (i = 0; i < fnlength; i++)
-                    *temp++ = Xchr(filename[i]);
-                sdone = 1;
-                break;
-
-            case '\0':
-                *temp++ = '%';
-                /* Back up to the null to force termination.  */
-                edit_value--;
-                break;
-
-            default:
-                *temp++ = '%';
-                *temp++ = c;
-                break;
-            }
-        } else
-            *temp++ = c;
-    }
-
-    *temp = 0;
-
-    /* Execute the command.  */
-#ifdef WIN32
-    /* Win32 reimplementation of the system() command
-       provides opportunity to call it asynchronously */
-    if (win32_system(command, true) != 0)
-#else
-    if (system(command) != 0)
-#endif
-        fprintf(stderr, "! Trouble executing `%s'.\n", command);
-
-    /* Quit, since we found an error.  */
-    uexit(1);
-}
 
 /* Read and write dump files.  As distributed, these files are
    architecture dependent; specifically, BigEndian and LittleEndian
@@ -956,16 +891,19 @@ calledit(packedASCIIcode * filename,
 /* This macro is always invoked as a statement.  It assumes a variable
    `temp'.  */
 
-#  define SWAP(x, y) temp = (x); (x) = (y); (y) = temp
+#  define SWAP(x, y) do { temp = x; x = y; y = temp; } while (0)
 
 
 /* Make the NITEMS items pointed at by P, each of size SIZE, be the
    opposite-endianness of whatever they are now.  */
 
-static void swap_items(char *p, int nitems, int size)
+void swap_items(char *pp, int nitems, int size)
 {
     char temp;
-
+    unsigned total = (unsigned) (nitems * size);
+    char *q = xmalloc(total);
+    char *p = q;
+    memcpy(p,pp,total);
     /* Since `size' does not change, we can write a while loop for each
        case, and avoid testing `size' for each time.  */
     switch (size) {
@@ -985,6 +923,18 @@ static void swap_items(char *p, int nitems, int size)
         }
         break;
 
+    case 12:
+        while (nitems--) {
+            SWAP(p[0], p[11]);
+            SWAP(p[1], p[10]);
+            SWAP(p[2], p[9]);
+            SWAP(p[3], p[8]);
+            SWAP(p[4], p[7]);
+            SWAP(p[5], p[6]);
+            p += size;
+        }
+        break;
+
     case 8:
         while (nitems--) {
             SWAP(p[0], p[7]);
@@ -1017,69 +967,11 @@ static void swap_items(char *p, int nitems, int size)
     default:
         FATAL1("Can't swap a %d-byte item for (un)dumping", size);
     }
+    memcpy(pp,q,total);
+    xfree(q); 
 }
 #endif                          /* not WORDS_BIGENDIAN and not NO_DUMP_SHARE */
 
-
-/* Here we write NITEMS items, each item being ITEM_SIZE bytes long.
-   The pointer to the stuff to write is P, and we write to the file
-   OUT_FILE.  */
-
-void do_dump(char *p, int item_size, int nitems, FILE * out_file)
-{
-#if !defined (WORDS_BIGENDIAN) && !defined (NO_DUMP_SHARE)
-    swap_items(p, nitems, item_size);
-#endif
-
-    if (fwrite(p, item_size, nitems, out_file) != (unsigned) nitems) {
-        fprintf(stderr, "! Could not write %d %d-byte item(s).\n",
-                nitems, item_size);
-        uexit(1);
-    }
-
-    /* Have to restore the old contents of memory, since some of it might
-       get used again.  */
-#if !defined (WORDS_BIGENDIAN) && !defined (NO_DUMP_SHARE)
-    swap_items(p, nitems, item_size);
-#endif
-}
-
-
-/* Here is the dual of the writing routine.  */
-
-void do_undump(char *p, int item_size, int nitems, FILE * in_file)
-{
-    if (fread(p, item_size, nitems, in_file) != (size_t) nitems)
-        FATAL2("Could not undump %d %d-byte item(s)", nitems, item_size);
-
-#if !defined (WORDS_BIGENDIAN) && !defined (NO_DUMP_SHARE)
-    swap_items(p, nitems, item_size);
-#endif
-}
-
-/* Look up VAR_NAME in texmf.cnf; assign either the value found there or
-   DFLT to *VAR.  */
-
-void setupboundvariable(int * var, const_string var_name, int dflt)
-{
-    string expansion = kpse_var_value(var_name);
-    *var = dflt;
-
-    if (expansion) {
-        int conf_val = atoi(expansion);
-        /* It's ok if the cnf file specifies 0 for extra_mem_{top,bot}, etc.
-           But negative numbers are always wrong.  */
-        if (conf_val < 0 || (conf_val == 0 && dflt > 0)) {
-            fprintf(stderr,
-                    "%s: Bad value (%ld) in texmf.cnf for %s, keeping %ld.\n",
-                    program_invocation_name,
-                    (long) conf_val, var_name + 1, (long) dflt);
-        } else {
-            *var = conf_val;    /* We'll make further checks later.  */
-        }
-        free(expansion);
-    }
-}
 
 
 /* Get the job name to be used, which may have been set from the
