@@ -45,8 +45,6 @@ typedef void* voidpointer;
 #endif
 #endif
 
-#ifndef luaTeX /* everything */
-
 /* Some things are the same except for the name.  */
 #ifdef TeX
 #if defined (pdfTeX)
@@ -61,6 +59,22 @@ typedef void* voidpointer;
 #elif defined (Aleph)
 #define TEXMFPOOLNAME "aleph.pool"
 #define TEXMFENGINENAME "aleph"
+#elif defined (pTeX)
+#define TEXMFPOOLNAME "ptex.pool"
+#define TEXMFENGINENAME "ptex"
+#include "ptexdir/kanji.h"
+#elif defined (epTeX)
+#define TEXMFPOOLNAME "eptex.pool"
+#define TEXMFENGINENAME "eptex"
+#include "ptexdir/kanji.h"
+#elif defined (upTeX)
+#define TEXMFPOOLNAME "uptex.pool"
+#define TEXMFENGINENAME "uptex"
+#include "uptexdir/kanji.h"
+#elif defined (eupTeX)
+#define TEXMFPOOLNAME "euptex.pool"
+#define TEXMFENGINENAME "euptex"
+#include "uptexdir/kanji.h"
 #else
 #define TEXMFPOOLNAME "tex.pool"
 #define TEXMFENGINENAME "tex"
@@ -81,12 +95,6 @@ typedef void* voidpointer;
 #define OUT_FILE gffile
 #define OUT_BUF gfbuf
 #endif /* MF */
-#ifdef MP
-#define TEXMFPOOLNAME "mp.pool"
-#define TEXMFENGINENAME "metapost"
-#define DUMP_FILE memfile
-#define DUMP_FORMAT kpse_mem_format
-#endif /* MP */
 
 /* Restore underscores.  */
 #define kpsedvipsconfigformat kpse_dvips_config_format
@@ -97,25 +105,27 @@ typedef void* voidpointer;
 #define kpsetexpoolformat kpse_texpool_format
 #define kpsetexformat kpse_tex_format
 
-/* Hacks for TeX that are better not to #ifdef, see texmfmp.c.  */
+/* Hacks for TeX that are better not to #ifdef, see lib/openclose.c.  */
 extern int tfmtemp, texinputtype;
 
-/* TeX, MF and MetaPost use this.  */
-extern boolean openinnameok (const_string);
-extern boolean openoutnameok (const_string);
-
 /* pdfTeX uses these for pipe support */
 #if defined(pdfTeX)
 extern boolean open_in_or_pipe (FILE **, int, const_string fopen_mode);
 extern boolean open_out_or_pipe (FILE **, const_string fopen_mode);
 extern void close_file_or_pipe (FILE *);
+#define ENABLE_PIPES 1
+#else
+#define ENABLE_PIPES 0
 #endif
 
 /* Executing shell commands.  */
-extern void mk_shellcmdlist (char *);
-extern void init_shell_escape (void);
-extern int shell_cmd_is_allowed (char **cmd, char **safecmd, char **cmdname);
-extern int runsystem (char *cmd);
+extern int runsystem (const char *cmd);
+
+/* The entry point.  */
+extern void maininit (int ac, string *av);
+#if defined(WIN32) && defined(DLLPROC)
+extern __declspec(dllexport) int DLLPROC (int ac, string *av);
+#endif
 
 /* All but the Omega family use this. */
 #if !defined(Aleph)
@@ -153,13 +163,7 @@ extern void ipcpage (int);
 #endif /* IPC */
 #endif /* TeX */
 
-/* How to output to the GF or DVI file.  */
-#define WRITE_OUT(a, b)							\
-  if ((size_t) fwrite ((char *) &OUT_BUF[a], sizeof (OUT_BUF[a]),       \
-                    (size_t) ((size_t)(b) - (size_t)(a) + 1), OUT_FILE) \
-      != (size_t) ((size_t) (b) - (size_t) (a) + 1))                    \
-    FATAL_PERROR ("fwrite");
-
+/* How to flush the DVI file.  */
 #define flush_out() fflush (OUT_FILE)
 
 /* Used to write to a TFM file.  */
@@ -184,12 +188,11 @@ extern boolean input_line (FILE *);
 #define	dateandtime(i,j,k,l) get_date_and_time (&(i), &(j), &(k), &(l))
 extern void get_date_and_time (integer *, integer *, integer *, integer *);
 
+#if defined(pdfTeX)
 /* Get high-res time info. */
 #define secondsandmicros(i,j) get_seconds_and_micros (&(i), &(j))
 extern void get_seconds_and_micros (integer *, integer *);
-
-/* This routine has to return a scaled value. */
-extern integer getrandomseed (void);
+#endif
 
 /* Copy command-line arguments into the buffer, despite the name.  */
 extern void topenin (void);
@@ -200,13 +203,10 @@ extern void topenin (void);
    to the *coerce.h files. */
 /* extern void calledit (); */
 
-/* Set an array size from texmf.cnf.  */
-extern void setupboundvariable (integer *, const_string, integer);
-
 /* These defines reroute the file i/o calls to the new pipe-enabled 
    functions in texmfmp.c*/
 
-#if defined(pdfTeX)
+#if ENABLE_PIPES
 #undef aopenin
 #undef aopenout
 #undef aclose
@@ -225,15 +225,14 @@ extern void setupboundvariable (integer *, const_string, integer);
 #define bopenout(f)	open_output (&(f), FOPEN_WBIN_MODE)
 #define bclose		aclose
 #ifdef XeTeX
-/* f is declared as gzFile (typedef'd as void *), but we temporarily
-   use it for a FILE * so that we can use the standard open calls */
+/* f is declared as gzFile, but we temporarily use it for a FILE *
+   so that we can use the standard open calls */
 #define wopenin(f)	(open_input ((FILE**)&(f), DUMP_FORMAT, FOPEN_RBIN_MODE) \
 						&& (f = gzdopen(fileno((FILE*)f), FOPEN_RBIN_MODE)))
 #define wopenout(f)	(open_output ((FILE**)&(f), FOPEN_WBIN_MODE) \
 						&& (f = gzdopen(fileno((FILE*)f), FOPEN_WBIN_MODE)) \
 						&& (gzsetparams(f, 1, Z_DEFAULT_STRATEGY) == Z_OK))
 #define wclose(f)	gzclose(f)
-#define weof(f)		gzeof(f)
 #else
 #define wopenin(f)	open_input (&(f), DUMP_FORMAT, FOPEN_RBIN_MODE)
 #define wopenout	bopenout
@@ -251,10 +250,6 @@ extern void setupboundvariable (integer *, const_string, integer);
 #define dumpcore uexit (1)
 #endif
 
-#ifdef MP
-extern boolean callmakempx (string, string);
-#endif
-
 #ifdef MF
 extern boolean initscreen (void);
 extern void updatescreen (void);
@@ -285,7 +280,7 @@ extern void paintrow (/*screenrow, pixelcolor, transspec, screencol*/);
       if ((&(base))[i] < (low) || (&(base))[i] > (high)) {              \
         FATAL5 ("Item %u (=%ld) of .fmt array at %lx <%ld or >%ld",     \
                 i, (unsigned long) (&(base))[i], (unsigned long) &(base),\
-                (unsigned long) low, (integer) high);                   \
+                (unsigned long) low, (unsigned long) high);                   \
       }                                                                 \
     }									\
   } while (0)
@@ -301,7 +296,7 @@ extern void paintrow (/*screenrow, pixelcolor, transspec, screencol*/);
       if ((&(base))[i] > (high)) {              			\
         FATAL4 ("Item %u (=%ld) of .fmt array at %lx >%ld",     	\
                 i, (unsigned long) (&(base))[i], (unsigned long) &(base),\
-                (integer) high);                         		\
+                (unsigned long) high);                         		\
       }                                                                 \
     }									\
   } while (0)
@@ -352,128 +347,11 @@ extern void do_undump (char *, int, int, FILE *);
 #define	undumpint generic_undump
 #endif
 
+/* Handle SyncTeX, if requested */
+#if defined(TeX)
+# if defined(__SyncTeX__)
+#  include "synctexdir/synctex-common.h"
+extern char *generic_synctex_get_current_name(void);
+# endif
+#endif
 
-#else  /* this is for luaTeX */
-
-/* Some things are the same except for the name.  */
-
-#define TEXMFPOOLNAME "luatex.pool"
-#define TEXMFENGINENAME "luatex"
-
-#define DUMP_FILE fmt_file
-#define DUMP_FORMAT kpse_fmt_format
-#define write_dvi WRITE_OUT
-#define flush_dvi flush_out
-#define OUT_FILE dvi_file
-#define OUT_BUF dvi_buf
-
-/* Restore underscores.  */
-#define kpsetexformat kpse_tex_format
-#define mainbody main_body
-#define t_open_in topenin
-
-/* Hacks for TeX that are better not to #ifdef, see texmfmp.c.  */
-extern int tfmtemp, texinputtype;
-
-/* TeX, MF and MetaPost use this.  */
-extern boolean openinnameok (const_string);
-extern boolean openoutnameok (const_string);
-
-/* pdfTeX uses these for pipe support */
-extern boolean open_in_or_pipe (FILE **, int, const_string fopen_mode);
-extern boolean open_out_or_pipe (FILE **, const_string fopen_mode);
-extern void close_file_or_pipe (FILE *);
-
-/* Executing shell commands.  */
-extern void mk_shellcmdlist (char *);
-extern void init_shell_escape (void);
-extern int shell_cmd_is_allowed (char **cmd, char **safecmd, char **cmdname);
-extern int runsystem (char *cmd);
-
-#ifndef GLUERATIO_TYPE
-#define GLUERATIO_TYPE double
-#endif
-typedef GLUERATIO_TYPE glueratio;
-
-#if defined(__DJGPP__) && defined (IPC)
-#undef IPC
-#endif
-
-#ifdef IPC
-extern void ipcpage (int);
-#endif /* IPC */
-
-
-/* How to output to the GF or DVI file.  */
-#define	WRITE_OUT(a, b)							\
-  if (fwrite ((char *) &OUT_BUF[a], sizeof (OUT_BUF[a]),		\
-                 (int) ((b) - (a) + 1), OUT_FILE) 			\
-      != (int) ((b) - (a) + 1))						\
-    FATAL_PERROR ("fwrite");
-
-#define flush_out() fflush (OUT_FILE)
-
-/* Read a line of input as quickly as possible.  */
-#define	input_ln(stream, flag) input_line (stream)
-
-extern boolean input_line (FILE *);
-
-/* This routine has to return four values.  */
-#define	dateandtime(i,j,k,l) get_date_and_time (&(i), &(j), &(k), &(l))
-extern void get_date_and_time (integer *, integer *, integer *, integer *);
-
-/* Get high-res time info. */
-#define seconds_and_micros(i,j) get_seconds_and_micros (&(i), &(j))
-extern void get_seconds_and_micros (integer *, integer *);
-
-/* This routine has to return a scaled value. */
-extern integer getrandomseed (void);
-
-/* Copy command-line arguments into the buffer, despite the name.  */
-extern void topenin (void);
-
-/* Can't prototype this since it uses poolpointer and ASCIIcode, which
-   are defined later in mfd.h, and mfd.h uses stuff from here.  */
-/* Therefore the department of ugly hacks decided to move this declaration
-   to the *coerce.h files. */
-/* extern void calledit (); */
-
-/* Set an array size from texmf.cnf.  */
-extern void setupboundvariable (integer *, const_string, integer);
-
-/* These defines reroute the file i/o calls to the new pipe-enabled 
-   functions in texmfmp.c*/
-
-#undef aopenin
-#undef aopenout
-#undef aclose
-#define a_open_in(f,p)  open_in_or_pipe(&(f),p,FOPEN_RBIN_MODE)
-#define a_open_out(f)   open_out_or_pipe(&(f),FOPEN_W_MODE)
-#define a_close(f)     close_file_or_pipe(f)
-
-/* `bopenin' (and out) is used only for reading (and writing) .tfm
-   files; `wopenin' (and out) only for dump files.  The filenames are
-   passed in as a global variable, `nameoffile'.  */
-#define b_open_in(f)	open_input (&(f), kpse_tfm_format, FOPEN_RBIN_MODE)
-#define ocp_open_in(f)	open_input (&(f), kpse_ocp_format, FOPEN_RBIN_MODE)
-#define ofm_open_in(f)	open_input (&(f), kpse_ofm_format, FOPEN_RBIN_MODE)
-#define b_open_out(f)	open_output (&(f), FOPEN_WBIN_MODE)
-
-/* Used in tex.ch (section 1338) to get a core dump in debugging mode.  */
-#ifdef unix
-#define dumpcore abort
-#else
-#define dumpcore uexit (1)
-#endif
-
-#define b_close close_file
-/* We define the routines to do the actual work in texmf.c.  */
-#define w_open_in(f)     zopen_w_input (&(f), DUMP_FORMAT, FOPEN_RBIN_MODE)
-#define w_open_out(f)    zopen_w_output (&(f), FOPEN_WBIN_MODE)
-#define w_close         zwclose
-
-extern boolean zopen_w_input (FILE **, int, const_string fopen_mode);
-extern boolean zopen_w_output (FILE **, const_string fopen_mode);
-extern void zwclose (FILE *);
-
-#endif /* luaTeX */
